"""Meshtastic unit tests for tunnel.py"""
import logging
import re
import sys
from unittest.mock import MagicMock, patch

import pytest

from meshtastic import mt_config

from ..tcp_interface import TCPInterface
from ..tunnel import Tunnel, onTunnelReceive


@pytest.mark.unit
@patch("platform.system")
def test_Tunnel_on_non_linux_system(mock_platform_system):
    """Test that we cannot instantiate a Tunnel on a non Linux system"""
    a_mock = MagicMock()
    a_mock.return_value = "notLinux"
    mock_platform_system.side_effect = a_mock
    with patch("socket.socket") as mock_socket:
        with pytest.raises(Tunnel.TunnelError) as pytest_wrapped_e:
            iface = TCPInterface(hostname="localhost", noProto=True)
            Tunnel(iface)
        assert pytest_wrapped_e.type == Tunnel.TunnelError
        assert mock_socket.called


@pytest.mark.unit
@patch("platform.system")
def test_Tunnel_without_interface(mock_platform_system):
    """Test that we can not instantiate a Tunnel without a valid interface"""
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with pytest.raises(Tunnel.TunnelError) as pytest_wrapped_e:
        Tunnel(None)
    assert pytest_wrapped_e.type == Tunnel.TunnelError


@pytest.mark.unitslow
@patch("platform.system")
def test_Tunnel_with_interface(mock_platform_system, caplog, iface_with_nodes):
    """Test that we can not instantiate a Tunnel without a valid interface"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.WARNING):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            assert tun == mt_config.tunnelInstance
            iface.close()
    assert re.search(r"Not creating a TapDevice()", caplog.text, re.MULTILINE)
    assert re.search(r"Not starting TUN reader", caplog.text, re.MULTILINE)
    assert re.search(r"Not sending packet", caplog.text, re.MULTILINE)


@pytest.mark.unitslow
@patch("platform.system")
def test_onTunnelReceive_from_ourselves(mock_platform_system, caplog, iface_with_nodes):
    """Test onTunnelReceive"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    sys.argv = [""]
    mt_config.args = sys.argv
    packet = {"decoded": {"payload": "foo"}, "from": 2475227164}
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            mt_config.tunnelInstance = tun
            onTunnelReceive(packet, iface)
    assert re.search(r"in onTunnelReceive", caplog.text, re.MULTILINE)
    assert re.search(r"Ignoring message we sent", caplog.text, re.MULTILINE)


@pytest.mark.unit
@patch("platform.system")
def test_onTunnelReceive_from_someone_else(
    mock_platform_system, caplog, iface_with_nodes
):
    """Test onTunnelReceive"""
    iface = iface_with_nodes
    iface.myInfo.my_node_num = 2475227164
    sys.argv = [""]
    mt_config.args = sys.argv
    packet = {"decoded": {"payload": "foo"}, "from": 123}
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            mt_config.tunnelInstance = tun
            onTunnelReceive(packet, iface)
    assert re.search(r"in onTunnelReceive", caplog.text, re.MULTILINE)


@pytest.mark.unitslow
@patch("platform.system")
def test_shouldFilterPacket_random(mock_platform_system, caplog, iface_with_nodes):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # random packet
    packet = b"1234567890123456789012345678901234567890"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert not ignore


@pytest.mark.unitslow
@patch("platform.system")
def test_shouldFilterPacket_in_blacklist(
    mock_platform_system, caplog, iface_with_nodes
):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked IGMP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert ignore


@pytest.mark.unitslow
@patch("platform.system")
def test_shouldFilterPacket_icmp(mock_platform_system, caplog, iface_with_nodes):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked ICMP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert re.search(r"forwarding ICMP message", caplog.text, re.MULTILINE)
            assert not ignore


@pytest.mark.unit
@patch("platform.system")
def test_shouldFilterPacket_udp(mock_platform_system, caplog, iface_with_nodes):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked UDP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x11\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert re.search(r"forwarding udp", caplog.text, re.MULTILINE)
            assert not ignore


@pytest.mark.unitslow
@patch("platform.system")
def test_shouldFilterPacket_udp_blacklisted(
    mock_platform_system, caplog, iface_with_nodes
):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked UDP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x11\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x07\x6c\x07\x6c\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    # Note: custom logging level
    LOG_TRACE = 5
    with caplog.at_level(LOG_TRACE):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert re.search(r"ignoring blacklisted UDP", caplog.text, re.MULTILINE)
            assert ignore


@pytest.mark.unit
@patch("platform.system")
def test_shouldFilterPacket_tcp(mock_platform_system, caplog, iface_with_nodes):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked TCP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x06\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert re.search(r"forwarding tcp", caplog.text, re.MULTILINE)
            assert not ignore


@pytest.mark.unitslow
@patch("platform.system")
def test_shouldFilterPacket_tcp_blacklisted(
    mock_platform_system, caplog, iface_with_nodes
):
    """Test _shouldFilterPacket()"""
    iface = iface_with_nodes
    iface.noProto = True
    # faked TCP
    packet = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x06\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x17\x0c\x17\x0c\x00\x00\x00"
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    # Note: custom logging level
    LOG_TRACE = 5
    with caplog.at_level(LOG_TRACE):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            ignore = tun._shouldFilterPacket(packet)
            assert re.search(r"ignoring blacklisted TCP", caplog.text, re.MULTILINE)
            assert ignore


@pytest.mark.unitslow
@patch("platform.system")
def test_ipToNodeId_none(mock_platform_system, caplog, iface_with_nodes):
    """Test _ipToNodeId()"""
    iface = iface_with_nodes
    iface.noProto = True
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            nodeid = tun._ipToNodeId("something not useful")
            assert nodeid is None


@pytest.mark.unitslow
@patch("platform.system")
def test_ipToNodeId_all(mock_platform_system, caplog, iface_with_nodes):
    """Test _ipToNodeId()"""
    iface = iface_with_nodes
    iface.noProto = True
    a_mock = MagicMock()
    a_mock.return_value = "Linux"
    mock_platform_system.side_effect = a_mock
    with caplog.at_level(logging.DEBUG):
        with patch("socket.socket"):
            tun = Tunnel(iface)
            nodeid = tun._ipToNodeId(b"\x00\x00\xff\xff")
            assert nodeid == "^all"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/module_config.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'meshtastic/protobuf/module_config.proto\x12\x13meshtastic.protobuf\"\xfa&\n\x0cModuleConfig\x12<\n\x04mqtt\x18\x01 \x01(\x0b\x32,.meshtastic.protobuf.ModuleConfig.MQTTConfigH\x00\x12@\n\x06serial\x18\x02 \x01(\x0b\x32..meshtastic.protobuf.ModuleConfig.SerialConfigH\x00\x12]\n\x15\x65xternal_notification\x18\x03 \x01(\x0b\x32<.meshtastic.protobuf.ModuleConfig.ExternalNotificationConfigH\x00\x12M\n\rstore_forward\x18\x04 \x01(\x0b\x32\x34.meshtastic.protobuf.ModuleConfig.StoreForwardConfigH\x00\x12G\n\nrange_test\x18\x05 \x01(\x0b\x32\x31.meshtastic.protobuf.ModuleConfig.RangeTestConfigH\x00\x12\x46\n\ttelemetry\x18\x06 \x01(\x0b\x32\x31.meshtastic.protobuf.ModuleConfig.TelemetryConfigH\x00\x12O\n\x0e\x63\x61nned_message\x18\x07 \x01(\x0b\x32\x35.meshtastic.protobuf.ModuleConfig.CannedMessageConfigH\x00\x12>\n\x05\x61udio\x18\x08 \x01(\x0b\x32-.meshtastic.protobuf.ModuleConfig.AudioConfigH\x00\x12Q\n\x0fremote_hardware\x18\t \x01(\x0b\x32\x36.meshtastic.protobuf.ModuleConfig.RemoteHardwareConfigH\x00\x12M\n\rneighbor_info\x18\n \x01(\x0b\x32\x34.meshtastic.protobuf.ModuleConfig.NeighborInfoConfigH\x00\x12S\n\x10\x61mbient_lighting\x18\x0b \x01(\x0b\x32\x37.meshtastic.protobuf.ModuleConfig.AmbientLightingConfigH\x00\x12S\n\x10\x64\x65tection_sensor\x18\x0c \x01(\x0b\x32\x37.meshtastic.protobuf.ModuleConfig.DetectionSensorConfigH\x00\x12H\n\npaxcounter\x18\r \x01(\x0b\x32\x32.meshtastic.protobuf.ModuleConfig.PaxcounterConfigH\x00\x1a\xb9\x02\n\nMQTTConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t\x12\x10\n\x08username\x18\x03 \x01(\t\x12\x10\n\x08password\x18\x04 \x01(\t\x12\x1a\n\x12\x65ncryption_enabled\x18\x05 \x01(\x08\x12\x14\n\x0cjson_enabled\x18\x06 \x01(\x08\x12\x13\n\x0btls_enabled\x18\x07 \x01(\x08\x12\x0c\n\x04root\x18\x08 \x01(\t\x12\x1f\n\x17proxy_to_client_enabled\x18\t \x01(\x08\x12\x1d\n\x15map_reporting_enabled\x18\n \x01(\x08\x12P\n\x13map_report_settings\x18\x0b \x01(\x0b\x32\x33.meshtastic.protobuf.ModuleConfig.MapReportSettings\x1aN\n\x11MapReportSettings\x12\x1d\n\x15publish_interval_secs\x18\x01 \x01(\r\x12\x1a\n\x12position_precision\x18\x02 \x01(\r\x1a\x8b\x01\n\x14RemoteHardwareConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\"\n\x1a\x61llow_undefined_pin_access\x18\x02 \x01(\x08\x12>\n\x0e\x61vailable_pins\x18\x03 \x03(\x0b\x32&.meshtastic.protobuf.RemoteHardwarePin\x1aZ\n\x12NeighborInfoConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x17\n\x0fupdate_interval\x18\x02 \x01(\r\x12\x1a\n\x12transmit_over_lora\x18\x03 \x01(\x08\x1a\xa0\x03\n\x15\x44\x65tectionSensorConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x1e\n\x16minimum_broadcast_secs\x18\x02 \x01(\r\x12\x1c\n\x14state_broadcast_secs\x18\x03 \x01(\r\x12\x11\n\tsend_bell\x18\x04 \x01(\x08\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x13\n\x0bmonitor_pin\x18\x06 \x01(\r\x12\x63\n\x16\x64\x65tection_trigger_type\x18\x07 \x01(\x0e\x32\x43.meshtastic.protobuf.ModuleConfig.DetectionSensorConfig.TriggerType\x12\x12\n\nuse_pullup\x18\x08 \x01(\x08\"\x88\x01\n\x0bTriggerType\x12\r\n\tLOGIC_LOW\x10\x00\x12\x0e\n\nLOGIC_HIGH\x10\x01\x12\x10\n\x0c\x46\x41LLING_EDGE\x10\x02\x12\x0f\n\x0bRISING_EDGE\x10\x03\x12\x1a\n\x16\x45ITHER_EDGE_ACTIVE_LOW\x10\x04\x12\x1b\n\x17\x45ITHER_EDGE_ACTIVE_HIGH\x10\x05\x1a\xed\x02\n\x0b\x41udioConfig\x12\x16\n\x0e\x63odec2_enabled\x18\x01 \x01(\x08\x12\x0f\n\x07ptt_pin\x18\x02 \x01(\r\x12I\n\x07\x62itrate\x18\x03 \x01(\x0e\x32\x38.meshtastic.protobuf.ModuleConfig.AudioConfig.Audio_Baud\x12\x0e\n\x06i2s_ws\x18\x04 \x01(\r\x12\x0e\n\x06i2s_sd\x18\x05 \x01(\r\x12\x0f\n\x07i2s_din\x18\x06 \x01(\r\x12\x0f\n\x07i2s_sck\x18\x07 \x01(\r\"\xa7\x01\n\nAudio_Baud\x12\x12\n\x0e\x43ODEC2_DEFAULT\x10\x00\x12\x0f\n\x0b\x43ODEC2_3200\x10\x01\x12\x0f\n\x0b\x43ODEC2_2400\x10\x02\x12\x0f\n\x0b\x43ODEC2_1600\x10\x03\x12\x0f\n\x0b\x43ODEC2_1400\x10\x04\x12\x0f\n\x0b\x43ODEC2_1300\x10\x05\x12\x0f\n\x0b\x43ODEC2_1200\x10\x06\x12\x0e\n\nCODEC2_700\x10\x07\x12\x0f\n\x0b\x43ODEC2_700B\x10\x08\x1av\n\x10PaxcounterConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\"\n\x1apaxcounter_update_interval\x18\x02 \x01(\r\x12\x16\n\x0ewifi_threshold\x18\x03 \x01(\x05\x12\x15\n\rble_threshold\x18\x04 \x01(\x05\x1a\x80\x05\n\x0cSerialConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x0c\n\x04\x65\x63ho\x18\x02 \x01(\x08\x12\x0b\n\x03rxd\x18\x03 \x01(\r\x12\x0b\n\x03txd\x18\x04 \x01(\r\x12H\n\x04\x62\x61ud\x18\x05 \x01(\x0e\x32:.meshtastic.protobuf.ModuleConfig.SerialConfig.Serial_Baud\x12\x0f\n\x07timeout\x18\x06 \x01(\r\x12H\n\x04mode\x18\x07 \x01(\x0e\x32:.meshtastic.protobuf.ModuleConfig.SerialConfig.Serial_Mode\x12$\n\x1coverride_console_serial_port\x18\x08 \x01(\x08\"\x8a\x02\n\x0bSerial_Baud\x12\x10\n\x0c\x42\x41UD_DEFAULT\x10\x00\x12\x0c\n\x08\x42\x41UD_110\x10\x01\x12\x0c\n\x08\x42\x41UD_300\x10\x02\x12\x0c\n\x08\x42\x41UD_600\x10\x03\x12\r\n\tBAUD_1200\x10\x04\x12\r\n\tBAUD_2400\x10\x05\x12\r\n\tBAUD_4800\x10\x06\x12\r\n\tBAUD_9600\x10\x07\x12\x0e\n\nBAUD_19200\x10\x08\x12\x0e\n\nBAUD_38400\x10\t\x12\x0e\n\nBAUD_57600\x10\n\x12\x0f\n\x0b\x42\x41UD_115200\x10\x0b\x12\x0f\n\x0b\x42\x41UD_230400\x10\x0c\x12\x0f\n\x0b\x42\x41UD_460800\x10\r\x12\x0f\n\x0b\x42\x41UD_576000\x10\x0e\x12\x0f\n\x0b\x42\x41UD_921600\x10\x0f\"_\n\x0bSerial_Mode\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\n\n\x06SIMPLE\x10\x01\x12\t\n\x05PROTO\x10\x02\x12\x0b\n\x07TEXTMSG\x10\x03\x12\x08\n\x04NMEA\x10\x04\x12\x0b\n\x07\x43\x41LTOPO\x10\x05\x12\x08\n\x04WS85\x10\x06\x1a\xe9\x02\n\x1a\x45xternalNotificationConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x11\n\toutput_ms\x18\x02 \x01(\r\x12\x0e\n\x06output\x18\x03 \x01(\r\x12\x14\n\x0coutput_vibra\x18\x08 \x01(\r\x12\x15\n\routput_buzzer\x18\t \x01(\r\x12\x0e\n\x06\x61\x63tive\x18\x04 \x01(\x08\x12\x15\n\ralert_message\x18\x05 \x01(\x08\x12\x1b\n\x13\x61lert_message_vibra\x18\n \x01(\x08\x12\x1c\n\x14\x61lert_message_buzzer\x18\x0b \x01(\x08\x12\x12\n\nalert_bell\x18\x06 \x01(\x08\x12\x18\n\x10\x61lert_bell_vibra\x18\x0c \x01(\x08\x12\x19\n\x11\x61lert_bell_buzzer\x18\r \x01(\x08\x12\x0f\n\x07use_pwm\x18\x07 \x01(\x08\x12\x13\n\x0bnag_timeout\x18\x0e \x01(\r\x12\x19\n\x11use_i2s_as_buzzer\x18\x0f \x01(\x08\x1a\x97\x01\n\x12StoreForwardConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x11\n\theartbeat\x18\x02 \x01(\x08\x12\x0f\n\x07records\x18\x03 \x01(\r\x12\x1a\n\x12history_return_max\x18\x04 \x01(\r\x12\x1d\n\x15history_return_window\x18\x05 \x01(\r\x12\x11\n\tis_server\x18\x06 \x01(\x08\x1a@\n\x0fRangeTestConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x0e\n\x06sender\x18\x02 \x01(\r\x12\x0c\n\x04save\x18\x03 \x01(\x08\x1a\xc9\x03\n\x0fTelemetryConfig\x12\x1e\n\x16\x64\x65vice_update_interval\x18\x01 \x01(\r\x12#\n\x1b\x65nvironment_update_interval\x18\x02 \x01(\r\x12\'\n\x1f\x65nvironment_measurement_enabled\x18\x03 \x01(\x08\x12\"\n\x1a\x65nvironment_screen_enabled\x18\x04 \x01(\x08\x12&\n\x1e\x65nvironment_display_fahrenheit\x18\x05 \x01(\x08\x12\x1b\n\x13\x61ir_quality_enabled\x18\x06 \x01(\x08\x12\x1c\n\x14\x61ir_quality_interval\x18\x07 \x01(\r\x12!\n\x19power_measurement_enabled\x18\x08 \x01(\x08\x12\x1d\n\x15power_update_interval\x18\t \x01(\r\x12\x1c\n\x14power_screen_enabled\x18\n \x01(\x08\x12\"\n\x1ahealth_measurement_enabled\x18\x0b \x01(\x08\x12\x1e\n\x16health_update_interval\x18\x0c \x01(\r\x12\x1d\n\x15health_screen_enabled\x18\r \x01(\x08\x1a\xf1\x04\n\x13\x43\x61nnedMessageConfig\x12\x17\n\x0frotary1_enabled\x18\x01 \x01(\x08\x12\x19\n\x11inputbroker_pin_a\x18\x02 \x01(\r\x12\x19\n\x11inputbroker_pin_b\x18\x03 \x01(\r\x12\x1d\n\x15inputbroker_pin_press\x18\x04 \x01(\r\x12\x62\n\x14inputbroker_event_cw\x18\x05 \x01(\x0e\x32\x44.meshtastic.protobuf.ModuleConfig.CannedMessageConfig.InputEventChar\x12\x63\n\x15inputbroker_event_ccw\x18\x06 \x01(\x0e\x32\x44.meshtastic.protobuf.ModuleConfig.CannedMessageConfig.InputEventChar\x12\x65\n\x17inputbroker_event_press\x18\x07 \x01(\x0e\x32\x44.meshtastic.protobuf.ModuleConfig.CannedMessageConfig.InputEventChar\x12\x17\n\x0fupdown1_enabled\x18\x08 \x01(\x08\x12\x0f\n\x07\x65nabled\x18\t \x01(\x08\x12\x1a\n\x12\x61llow_input_source\x18\n \x01(\t\x12\x11\n\tsend_bell\x18\x0b \x01(\x08\"c\n\x0eInputEventChar\x12\x08\n\x04NONE\x10\x00\x12\x06\n\x02UP\x10\x11\x12\x08\n\x04\x44OWN\x10\x12\x12\x08\n\x04LEFT\x10\x13\x12\t\n\x05RIGHT\x10\x14\x12\n\n\x06SELECT\x10\n\x12\x08\n\x04\x42\x41\x43K\x10\x1b\x12\n\n\x06\x43\x41NCEL\x10\x18\x1a\x65\n\x15\x41mbientLightingConfig\x12\x11\n\tled_state\x18\x01 \x01(\x08\x12\x0f\n\x07\x63urrent\x18\x02 \x01(\r\x12\x0b\n\x03red\x18\x03 \x01(\r\x12\r\n\x05green\x18\x04 \x01(\r\x12\x0c\n\x04\x62lue\x18\x05 \x01(\rB\x11\n\x0fpayload_variant\"m\n\x11RemoteHardwarePin\x12\x10\n\x08gpio_pin\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x38\n\x04type\x18\x03 \x01(\x0e\x32*.meshtastic.protobuf.RemoteHardwarePinType*I\n\x15RemoteHardwarePinType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0c\x44IGITAL_READ\x10\x01\x12\x11\n\rDIGITAL_WRITE\x10\x02\x42g\n\x13\x63om.geeksville.meshB\x12ModuleConfigProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.module_config_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\022ModuleConfigProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_REMOTEHARDWAREPINTYPE']._serialized_start=5164
  _globals['_REMOTEHARDWAREPINTYPE']._serialized_end=5237
  _globals['_MODULECONFIG']._serialized_start=65
  _globals['_MODULECONFIG']._serialized_end=5051
  _globals['_MODULECONFIG_MQTTCONFIG']._serialized_start=1080
  _globals['_MODULECONFIG_MQTTCONFIG']._serialized_end=1393
  _globals['_MODULECONFIG_MAPREPORTSETTINGS']._serialized_start=1395
  _globals['_MODULECONFIG_MAPREPORTSETTINGS']._serialized_end=1473
  _globals['_MODULECONFIG_REMOTEHARDWARECONFIG']._serialized_start=1476
  _globals['_MODULECONFIG_REMOTEHARDWARECONFIG']._serialized_end=1615
  _globals['_MODULECONFIG_NEIGHBORINFOCONFIG']._serialized_start=1617
  _globals['_MODULECONFIG_NEIGHBORINFOCONFIG']._serialized_end=1707
  _globals['_MODULECONFIG_DETECTIONSENSORCONFIG']._serialized_start=1710
  _globals['_MODULECONFIG_DETECTIONSENSORCONFIG']._serialized_end=2126
  _globals['_MODULECONFIG_DETECTIONSENSORCONFIG_TRIGGERTYPE']._serialized_start=1990
  _globals['_MODULECONFIG_DETECTIONSENSORCONFIG_TRIGGERTYPE']._serialized_end=2126
  _globals['_MODULECONFIG_AUDIOCONFIG']._serialized_start=2129
  _globals['_MODULECONFIG_AUDIOCONFIG']._serialized_end=2494
  _globals['_MODULECONFIG_AUDIOCONFIG_AUDIO_BAUD']._serialized_start=2327
  _globals['_MODULECONFIG_AUDIOCONFIG_AUDIO_BAUD']._serialized_end=2494
  _globals['_MODULECONFIG_PAXCOUNTERCONFIG']._serialized_start=2496
  _globals['_MODULECONFIG_PAXCOUNTERCONFIG']._serialized_end=2614
  _globals['_MODULECONFIG_SERIALCONFIG']._serialized_start=2617
  _globals['_MODULECONFIG_SERIALCONFIG']._serialized_end=3257
  _globals['_MODULECONFIG_SERIALCONFIG_SERIAL_BAUD']._serialized_start=2894
  _globals['_MODULECONFIG_SERIALCONFIG_SERIAL_BAUD']._serialized_end=3160
  _globals['_MODULECONFIG_SERIALCONFIG_SERIAL_MODE']._serialized_start=3162
  _globals['_MODULECONFIG_SERIALCONFIG_SERIAL_MODE']._serialized_end=3257
  _globals['_MODULECONFIG_EXTERNALNOTIFICATIONCONFIG']._serialized_start=3260
  _globals['_MODULECONFIG_EXTERNALNOTIFICATIONCONFIG']._serialized_end=3621
  _globals['_MODULECONFIG_STOREFORWARDCONFIG']._serialized_start=3624
  _globals['_MODULECONFIG_STOREFORWARDCONFIG']._serialized_end=3775
  _globals['_MODULECONFIG_RANGETESTCONFIG']._serialized_start=3777
  _globals['_MODULECONFIG_RANGETESTCONFIG']._serialized_end=3841
  _globals['_MODULECONFIG_TELEMETRYCONFIG']._serialized_start=3844
  _globals['_MODULECONFIG_TELEMETRYCONFIG']._serialized_end=4301
  _globals['_MODULECONFIG_CANNEDMESSAGECONFIG']._serialized_start=4304
  _globals['_MODULECONFIG_CANNEDMESSAGECONFIG']._serialized_end=4929
  _globals['_MODULECONFIG_CANNEDMESSAGECONFIG_INPUTEVENTCHAR']._serialized_start=4830
  _globals['_MODULECONFIG_CANNEDMESSAGECONFIG_INPUTEVENTCHAR']._serialized_end=4929
  _globals['_MODULECONFIG_AMBIENTLIGHTINGCONFIG']._serialized_start=4931
  _globals['_MODULECONFIG_AMBIENTLIGHTINGCONFIG']._serialized_end=5032
  _globals['_REMOTEHARDWAREPIN']._serialized_start=5053
  _globals['_REMOTEHARDWAREPIN']._serialized_end=5162
# @@protoc_insertion_point(module_scope)



"""code logging power consumption of meshtastic devices."""

import math
import time

from .power_supply import PowerSupply


class SimPowerSupply(PowerSupply):
    """A simulated power supply for testing."""

    def get_average_current_mA(self) -> float:
        """Returns average current of last measurement in mA (since last call to this method)"""

        # Sim a 20mW load that varies sinusoidally
        return (20.0 + 5 * math.sin(time.time()))



"""code logging power consumption of meshtastic devices."""

import math
from datetime import datetime


class PowerError(Exception):
    """An exception class for powermon errors"""

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class PowerMeter:
    """Abstract class for power meters."""

    def __init__(self):
        """Initialize the PowerMeter object."""
        self.prevPowerTime = datetime.now()

    def close(self) -> None:
        """Close the power meter."""

    def get_average_current_mA(self) -> float:
        """Returns average current of last measurement in mA (since last call to this method)"""
        return math.nan

    def get_min_current_mA(self):
        """Returns max current in mA (since last call to this method)."""
        # Subclasses must override for a better implementation
        return self.get_average_current_mA()

    def get_max_current_mA(self):
        """Returns max current in mA (since last call to this method)."""
        # Subclasses must override for a better implementation
        return self.get_average_current_mA()

    def reset_measurements(self):
        """Reset current measurements."""


class PowerSupply(PowerMeter):
    """Abstract class for power supplies."""

    def __init__(self):
        """Initialize the PowerSupply object."""
        super().__init__()
        self.v = 0.0

    def powerOn(self):
        """Turn on the power supply (using the voltage set in self.v)."""



"""Meshtastic test that the examples run as expected.
   We assume you have a python virtual environment in current directory.
   If not, you need to run: "python3 -m venv venv", "source venv/bin/activate", "pip install ."
"""
import subprocess

import pytest


@pytest.mark.examples
def test_examples_hello_world_serial_no_arg():
    """Test hello_world_serial without any args"""
    return_value, _ = subprocess.getstatusoutput(
        "source venv/bin/activate; python3 examples/hello_world_serial.py"
    )
    assert return_value == 3


@pytest.mark.examples
def test_examples_hello_world_serial_with_arg(capsys):
    """Test hello_world_serial with arg"""
    return_value, _ = subprocess.getstatusoutput(
        "source venv/bin/activate; python3 examples/hello_world_serial.py hello"
    )
    assert return_value == 1
    _, err = capsys.readouterr()
    assert err == ""
    # TODO: Why does this not work?
    # assert out == 'Warning: No Meshtastic devices detected.'



"""Meshtastic smoke tests a device setup with wifi.

   Need to have run the following on an esp32 device:
      meshtastic --set wifi_ssid 'foo' --set wifi_password 'sekret'
"""
import re
import subprocess

import pytest


@pytest.mark.smokewifi
def test_smokewifi_info():
    """Test --info"""
    return_value, out = subprocess.getstatusoutput(
        "meshtastic --info --host meshtastic.local"
    )
    assert re.search(r"^Owner", out, re.MULTILINE)
    assert re.search(r"^My info", out, re.MULTILINE)
    assert re.search(r"^Nodes in mesh", out, re.MULTILINE)
    assert re.search(r"^Preferences", out, re.MULTILINE)
    assert re.search(r"^Channels", out, re.MULTILINE)
    assert re.search(r"^  PRIMARY", out, re.MULTILINE)
    assert re.search(r"^Primary channel URL", out, re.MULTILINE)
    assert return_value == 0



"""Simple program to demo how to use meshtastic library.
   To run: python examples/hello_world_serial.py
"""

import sys

import meshtastic
import meshtastic.serial_interface

# simple arg check
if len(sys.argv) < 2:
    print(f"usage: {sys.argv[0]} message")
    sys.exit(3)

# By default will try to find a meshtastic device,
# otherwise provide a device path like /dev/ttyUSB0
iface = meshtastic.serial_interface.SerialInterface()
iface.sendText(sys.argv[1])
iface.close()



"""Node class
"""

import base64
import logging
import time

from typing import Optional, Union, List

from meshtastic.protobuf import admin_pb2, apponly_pb2, channel_pb2, localonly_pb2, mesh_pb2, portnums_pb2
from meshtastic.util import (
    Timeout,
    camel_to_snake,
    fromPSK,
    our_exit,
    pskToString,
    stripnl,
    message_to_json,
)


class Node:
    """A model of a (local or remote) node in the mesh

    Includes methods for localConfig, moduleConfig and channels
    """

    def __init__(self, iface, nodeNum, noProto=False, timeout: int = 300):
        """Constructor"""
        self.iface = iface
        self.nodeNum = nodeNum
        self.localConfig = localonly_pb2.LocalConfig()
        self.moduleConfig = localonly_pb2.LocalModuleConfig()
        self.channels = None
        self._timeout = Timeout(maxSecs=timeout)
        self.partialChannels: Optional[List] = None
        self.noProto = noProto
        self.cannedPluginMessage = None
        self.cannedPluginMessageMessages = None
        self.ringtone = None
        self.ringtonePart = None

        self.gotResponse = None

    def showChannels(self):
        """Show human readable description of our channels."""
        print("Channels:")
        if self.channels:
            logging.debug(f"self.channels:{self.channels}")
            for c in self.channels:
                cStr = message_to_json(c.settings)
                # don't show disabled channels
                if channel_pb2.Channel.Role.Name(c.role) != "DISABLED":
                    print(
                        f"  Index {c.index}: {channel_pb2.Channel.Role.Name(c.role)} psk={pskToString(c.settings.psk)} {cStr}"
                    )
        publicURL = self.getURL(includeAll=False)
        adminURL = self.getURL(includeAll=True)
        print(f"\nPrimary channel URL: {publicURL}")
        if adminURL != publicURL:
            print(f"Complete URL (includes all channels): {adminURL}")

    def showInfo(self):
        """Show human readable description of our node"""
        prefs = ""
        if self.localConfig:
            prefs = message_to_json(self.localConfig, multiline=True)
        print(f"Preferences: {prefs}\n")
        prefs = ""
        if self.moduleConfig:
            prefs = message_to_json(self.moduleConfig, multiline=True)
        print(f"Module preferences: {prefs}\n")
        self.showChannels()

    def setChannels(self, channels):
        """Set the channels for this node"""
        self.channels = channels
        self._fixupChannels()

    def requestChannels(self, startingIndex: int = 0):
        """Send regular MeshPackets to ask channels."""
        logging.debug(f"requestChannels for nodeNum:{self.nodeNum}")
        # only initialize if we're starting out fresh
        if startingIndex == 0:
            self.channels = None
            self.partialChannels = []  # We keep our channels in a temp array until finished
        self._requestChannel(startingIndex)

    def onResponseRequestSettings(self, p):
        """Handle the response packets for requesting settings _requestSettings()"""
        logging.debug(f"onResponseRequestSetting() p:{p}")
        config_values = None
        if "routing" in p["decoded"]:
            if p["decoded"]["routing"]["errorReason"] != "NONE":
                print(f'Error on response: {p["decoded"]["routing"]["errorReason"]}')
                self.iface._acknowledgment.receivedNak = True
        else:
            self.iface._acknowledgment.receivedAck = True
            print("")
            adminMessage = p["decoded"]["admin"]
            if "getConfigResponse" in adminMessage:
                oneof = "get_config_response"
                resp = adminMessage["getConfigResponse"]
                field = list(resp.keys())[0]
                config_type = self.localConfig.DESCRIPTOR.fields_by_name.get(
                    camel_to_snake(field)
                )
                if config_type is not None:
                    config_values = getattr(self.localConfig, config_type.name)
            elif "getModuleConfigResponse" in adminMessage:
                oneof = "get_module_config_response"
                resp = adminMessage["getModuleConfigResponse"]
                field = list(resp.keys())[0]
                config_type = self.moduleConfig.DESCRIPTOR.fields_by_name.get(
                    camel_to_snake(field)
                )
                config_values = getattr(self.moduleConfig, config_type.name)
            else:
                print(
                    "Did not receive a valid response. Make sure to have a shared channel named 'admin'."
                )
                return
            if config_values is not None:
                raw_config = getattr(getattr(adminMessage['raw'], oneof), camel_to_snake(field))
                config_values.CopyFrom(raw_config)
                print(f"{str(camel_to_snake(field))}:\n{str(config_values)}")

    def requestConfig(self, configType):
        """Request the config from the node via admin message"""
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onResponseRequestSettings
            print("Requesting current config from remote node (this can take a while).")
        p = admin_pb2.AdminMessage()
        if isinstance(configType, int):
            p.get_config_request = configType

        else:
            msgIndex = configType.index
            if configType.containing_type.name == "LocalConfig":
                p.get_config_request = msgIndex
            else:
                p.get_module_config_request = msgIndex

        self._sendAdmin(p, wantResponse=True, onResponse=onResponse)
        if onResponse:
            self.iface.waitForAckNak()

    def turnOffEncryptionOnPrimaryChannel(self):
        """Turn off encryption on primary channel."""
        self.channels[0].settings.psk = fromPSK("none")
        print("Writing modified channels to device")
        self.writeChannel(0)

    def waitForConfig(self, attribute="channels"):
        """Block until radio config is received. Returns True if config has been received."""
        return self._timeout.waitForSet(self, attrs=("localConfig", attribute))

    def writeConfig(self, config_name):
        """Write the current (edited) localConfig to the device"""
        if self.localConfig is None:
            our_exit("Error: No localConfig has been read")

        p = admin_pb2.AdminMessage()

        if config_name == "device":
            p.set_config.device.CopyFrom(self.localConfig.device)
        elif config_name == "position":
            p.set_config.position.CopyFrom(self.localConfig.position)
        elif config_name == "power":
            p.set_config.power.CopyFrom(self.localConfig.power)
        elif config_name == "network":
            p.set_config.network.CopyFrom(self.localConfig.network)
        elif config_name == "display":
            p.set_config.display.CopyFrom(self.localConfig.display)
        elif config_name == "lora":
            p.set_config.lora.CopyFrom(self.localConfig.lora)
        elif config_name == "bluetooth":
            p.set_config.bluetooth.CopyFrom(self.localConfig.bluetooth)
        elif config_name == "security":
            p.set_config.security.CopyFrom(self.localConfig.security)
        elif config_name == "mqtt":
            p.set_module_config.mqtt.CopyFrom(self.moduleConfig.mqtt)
        elif config_name == "serial":
            p.set_module_config.serial.CopyFrom(self.moduleConfig.serial)
        elif config_name == "external_notification":
            p.set_module_config.external_notification.CopyFrom(
                self.moduleConfig.external_notification
            )
        elif config_name == "store_forward":
            p.set_module_config.store_forward.CopyFrom(self.moduleConfig.store_forward)
        elif config_name == "range_test":
            p.set_module_config.range_test.CopyFrom(self.moduleConfig.range_test)
        elif config_name == "telemetry":
            p.set_module_config.telemetry.CopyFrom(self.moduleConfig.telemetry)
        elif config_name == "canned_message":
            p.set_module_config.canned_message.CopyFrom(
                self.moduleConfig.canned_message
            )
        elif config_name == "audio":
            p.set_module_config.audio.CopyFrom(self.moduleConfig.audio)
        elif config_name == "remote_hardware":
            p.set_module_config.remote_hardware.CopyFrom(
                self.moduleConfig.remote_hardware
            )
        elif config_name == "neighbor_info":
            p.set_module_config.neighbor_info.CopyFrom(self.moduleConfig.neighbor_info)
        elif config_name == "detection_sensor":
            p.set_module_config.detection_sensor.CopyFrom(self.moduleConfig.detection_sensor)
        elif config_name == "ambient_lighting":
            p.set_module_config.ambient_lighting.CopyFrom(self.moduleConfig.ambient_lighting)
        elif config_name == "paxcounter":
            p.set_module_config.paxcounter.CopyFrom(self.moduleConfig.paxcounter)
        else:
            our_exit(f"Error: No valid config with name {config_name}")

        logging.debug(f"Wrote: {config_name}")
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        self._sendAdmin(p, onResponse=onResponse)

    def writeChannel(self, channelIndex, adminIndex=0):
        """Write the current (edited) channel to the device"""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.set_channel.CopyFrom(self.channels[channelIndex])
        self._sendAdmin(p, adminIndex=adminIndex)
        logging.debug(f"Wrote channel {channelIndex}")

    def getChannelByChannelIndex(self, channelIndex):
        """Get channel by channelIndex
        channelIndex: number, typically 0-7; based on max number channels
        returns: None if there is no channel found
        """
        ch = None
        if self.channels and 0 <= channelIndex < len(self.channels):
            ch = self.channels[channelIndex]
        return ch

    def deleteChannel(self, channelIndex):
        """Delete the specified channelIndex and shift other channels up"""
        ch = self.channels[channelIndex]
        if ch.role not in (
            channel_pb2.Channel.Role.SECONDARY,
            channel_pb2.Channel.Role.DISABLED,
        ):
            our_exit("Warning: Only SECONDARY channels can be deleted")

        # we are careful here because if we move the "admin" channel the channelIndex we need to use
        # for sending admin channels will also change
        adminIndex = self.iface.localNode._getAdminChannelIndex()

        self.channels.pop(channelIndex)
        self._fixupChannels()  # expand back to 8 channels

        index = channelIndex
        while index < 8:
            self.writeChannel(index, adminIndex=adminIndex)
            index += 1

            # if we are updating the local node, we might end up
            # *moving* the admin channel index as we are writing
            if (self.iface.localNode == self) and index >= adminIndex:
                # We've now passed the old location for admin index
                # (and written it), so we can start finding it by name again
                adminIndex = 0

    def getChannelByName(self, name):
        """Try to find the named channel or return None"""
        for c in self.channels or []:
            if c.settings and c.settings.name == name:
                return c
        return None

    def getDisabledChannel(self):
        """Return the first channel that is disabled (i.e. available for some new use)"""
        for c in self.channels:
            if c.role == channel_pb2.Channel.Role.DISABLED:
                return c
        return None

    def _getAdminChannelIndex(self):
        """Return the channel number of the admin channel, or 0 if no reserved channel"""
        for c in self.channels or []:
            if c.settings and c.settings.name.lower() == "admin":
                return c.index
        return 0

    def setOwner(self, long_name: Optional[str]=None, short_name: Optional[str]=None, is_licensed: bool=False):
        """Set device owner name"""
        logging.debug(f"in setOwner nodeNum:{self.nodeNum}")
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()

        nChars = 4
        if long_name is not None:
            long_name = long_name.strip()
            p.set_owner.long_name = long_name
            p.set_owner.is_licensed = is_licensed
        if short_name is not None:
            short_name = short_name.strip()
            if len(short_name) > nChars:
                short_name = short_name[:nChars]
                print(f"Maximum is 4 characters, truncated to {short_name}")
            p.set_owner.short_name = short_name

        # Note: These debug lines are used in unit tests
        logging.debug(f"p.set_owner.long_name:{p.set_owner.long_name}:")
        logging.debug(f"p.set_owner.short_name:{p.set_owner.short_name}:")
        logging.debug(f"p.set_owner.is_licensed:{p.set_owner.is_licensed}")
        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def getURL(self, includeAll: bool = True):
        """The sharable URL that describes the current channel"""
        # Only keep the primary/secondary channels, assume primary is first
        channelSet = apponly_pb2.ChannelSet()
        if self.channels:
            for c in self.channels:
                if c.role == channel_pb2.Channel.Role.PRIMARY or (
                    includeAll and c.role == channel_pb2.Channel.Role.SECONDARY
                ):
                    channelSet.settings.append(c.settings)

        if len(self.localConfig.ListFields()) == 0:
            self.requestConfig(self.localConfig.DESCRIPTOR.fields_by_name.get('lora'))
        channelSet.lora_config.CopyFrom(self.localConfig.lora)
        some_bytes = channelSet.SerializeToString()
        s = base64.urlsafe_b64encode(some_bytes).decode("ascii")
        s = s.replace("=", "").replace("+", "-").replace("/", "_")
        return f"https://meshtastic.org/e/#{s}"

    def setURL(self, url: str, addOnly: bool = False):
        """Set mesh network URL"""
        if self.localConfig is None or self.channels is None:
            our_exit("Warning: config or channels not loaded")

        # URLs are of the form https://meshtastic.org/d/#{base64_channel_set}
        # Split on '/#' to find the base64 encoded channel settings
        if addOnly:
            splitURL = url.split("/?add=true#")
        else:
            splitURL = url.split("/#")
        if len(splitURL) == 1:
            our_exit(f"Warning: Invalid URL '{url}'")
        b64 = splitURL[-1]

        # We normally strip padding to make for a shorter URL, but the python parser doesn't like
        # that.  So add back any missing padding
        # per https://stackoverflow.com/a/9807138
        missing_padding = len(b64) % 4
        if missing_padding:
            b64 += "=" * (4 - missing_padding)

        decodedURL = base64.urlsafe_b64decode(b64)
        channelSet = apponly_pb2.ChannelSet()
        channelSet.ParseFromString(decodedURL)

        if len(channelSet.settings) == 0:
            our_exit("Warning: There were no settings.")

        if addOnly:
            # Add new channels with names not already present
            # Don't change existing channels
            for chs in channelSet.settings:
                channelExists = self.getChannelByName(chs.name)
                if channelExists or chs.name == "":
                    print(f"Ignoring existing or empty channel \"{chs.name}\" from add URL")
                    continue
                ch = self.getDisabledChannel()
                if not ch:
                    our_exit("Warning: No free channels were found")
                ch.settings.CopyFrom(chs)
                ch.role = channel_pb2.Channel.Role.SECONDARY
                print(f"Adding new channel '{chs.name}' to device")
                self.writeChannel(ch.index)
        else:
            i = 0
            for chs in channelSet.settings:
                ch = channel_pb2.Channel()
                ch.role = (
                    channel_pb2.Channel.Role.PRIMARY
                    if i == 0
                    else channel_pb2.Channel.Role.SECONDARY
                )
                ch.index = i
                ch.settings.CopyFrom(chs)
                self.channels[ch.index] = ch
                logging.debug(f"Channel i:{i} ch:{ch}")
                self.writeChannel(ch.index)
                i = i + 1

        p = admin_pb2.AdminMessage()
        p.set_config.lora.CopyFrom(channelSet.lora_config)
        self.ensureSessionKey()
        self._sendAdmin(p)

    def onResponseRequestRingtone(self, p):
        """Handle the response packet for requesting ringtone part 1"""
        logging.debug(f"onResponseRequestRingtone() p:{p}")
        errorFound = False
        if "routing" in p["decoded"]:
            if p["decoded"]["routing"]["errorReason"] != "NONE":
                errorFound = True
                print(f'Error on response: {p["decoded"]["routing"]["errorReason"]}')
        if errorFound is False:
            if "decoded" in p:
                if "admin" in p["decoded"]:
                    if "raw" in p["decoded"]["admin"]:
                        self.ringtonePart = p["decoded"]["admin"][
                            "raw"
                        ].get_ringtone_response
                        logging.debug(f"self.ringtonePart:{self.ringtonePart}")
                        self.gotResponse = True

    def get_ringtone(self):
        """Get the ringtone. Concatenate all pieces together and return a single string."""
        logging.debug(f"in get_ringtone()")
        if not self.ringtone:
            p1 = admin_pb2.AdminMessage()
            p1.get_ringtone_request = True
            self.gotResponse = False
            self._sendAdmin(
                p1, wantResponse=True, onResponse=self.onResponseRequestRingtone
            )
            while self.gotResponse is False:
                time.sleep(0.1)

            logging.debug(f"self.ringtone:{self.ringtone}")

            self.ringtone = ""
            if self.ringtonePart:
                self.ringtone += self.ringtonePart

        print(f"ringtone:{self.ringtone}")
        logging.debug(f"ringtone:{self.ringtone}")
        return self.ringtone

    def set_ringtone(self, ringtone):
        """Set the ringtone. The ringtone length must be less than 230 character."""

        if len(ringtone) > 230:
            our_exit("Warning: The ringtone must be less than 230 characters.")
        self.ensureSessionKey()
        # split into chunks
        chunks = []
        chunks_size = 230
        for i in range(0, len(ringtone), chunks_size):
            chunks.append(ringtone[i : i + chunks_size])

        # for each chunk, send a message to set the values
        # for i in range(0, len(chunks)):
        for i, chunk in enumerate(chunks):
            p = admin_pb2.AdminMessage()

            # TODO: should be a way to improve this
            if i == 0:
                p.set_ringtone_message = chunk

            logging.debug(f"Setting ringtone '{chunk}' part {i+1}")
            # If sending to a remote node, wait for ACK/NAK
            if self == self.iface.localNode:
                onResponse = None
            else:
                onResponse = self.onAckNak
            return self._sendAdmin(p, onResponse=onResponse)

    def onResponseRequestCannedMessagePluginMessageMessages(self, p):
        """Handle the response packet for requesting canned message plugin message part 1"""
        logging.debug(f"onResponseRequestCannedMessagePluginMessageMessages() p:{p}")
        errorFound = False
        if "routing" in p["decoded"]:
            if p["decoded"]["routing"]["errorReason"] != "NONE":
                errorFound = True
                print(f'Error on response: {p["decoded"]["routing"]["errorReason"]}')
        if errorFound is False:
            if "decoded" in p:
                if "admin" in p["decoded"]:
                    if "raw" in p["decoded"]["admin"]:
                        self.cannedPluginMessageMessages = p["decoded"]["admin"][
                            "raw"
                        ].get_canned_message_module_messages_response
                        logging.debug(
                            f"self.cannedPluginMessageMessages:{self.cannedPluginMessageMessages}"
                        )
                        self.gotResponse = True

    def get_canned_message(self):
        """Get the canned message string. Concatenate all pieces together and return a single string."""
        logging.debug(f"in get_canned_message()")
        if not self.cannedPluginMessage:
            p1 = admin_pb2.AdminMessage()
            p1.get_canned_message_module_messages_request = True
            self.gotResponse = False
            self._sendAdmin(
                p1,
                wantResponse=True,
                onResponse=self.onResponseRequestCannedMessagePluginMessageMessages,
            )
            while self.gotResponse is False:
                time.sleep(0.1)

            logging.debug(
                f"self.cannedPluginMessageMessages:{self.cannedPluginMessageMessages}"
            )

            self.cannedPluginMessage = ""
            if self.cannedPluginMessageMessages:
                self.cannedPluginMessage += self.cannedPluginMessageMessages

        print(f"canned_plugin_message:{self.cannedPluginMessage}")
        logging.debug(f"canned_plugin_message:{self.cannedPluginMessage}")
        return self.cannedPluginMessage

    def set_canned_message(self, message):
        """Set the canned message. The canned messages length must be less than 200 character."""

        if len(message) > 200:
            our_exit("Warning: The canned message must be less than 200 characters.")
        self.ensureSessionKey()
        # split into chunks
        chunks = []
        chunks_size = 200
        for i in range(0, len(message), chunks_size):
            chunks.append(message[i : i + chunks_size])

        # for each chunk, send a message to set the values
        # for i in range(0, len(chunks)):
        for i, chunk in enumerate(chunks):
            p = admin_pb2.AdminMessage()

            # TODO: should be a way to improve this
            if i == 0:
                p.set_canned_message_module_messages = chunk

            logging.debug(f"Setting canned message '{chunk}' part {i+1}")
            # If sending to a remote node, wait for ACK/NAK
            if self == self.iface.localNode:
                onResponse = None
            else:
                onResponse = self.onAckNak
            return self._sendAdmin(p, onResponse=onResponse)

    def exitSimulator(self):
        """Tell a simulator node to exit (this message
        is ignored for other nodes)"""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.exit_simulator = True
        logging.debug("in exitSimulator()")

        return self._sendAdmin(p)

    def reboot(self, secs: int = 10):
        """Tell the node to reboot."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.reboot_seconds = secs
        logging.info(f"Telling node to reboot in {secs} seconds")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def beginSettingsTransaction(self):
        """Tell the node to open a transaction to edit settings."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.begin_edit_settings = True
        logging.info(f"Telling open a transaction to edit settings")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def commitSettingsTransaction(self):
        """Tell the node to commit the open transaction for editing settings."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.commit_edit_settings = True
        logging.info(f"Telling node to commit open transaction for editing settings")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def rebootOTA(self, secs: int = 10):
        """Tell the node to reboot into factory firmware."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.reboot_ota_seconds = secs
        logging.info(f"Telling node to reboot to OTA in {secs} seconds")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def enterDFUMode(self):
        """Tell the node to enter DFU mode (NRF52)."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.enter_dfu_mode_request = True
        logging.info(f"Telling node to enable DFU mode")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def shutdown(self, secs: int = 10):
        """Tell the node to shutdown."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.shutdown_seconds = secs
        logging.info(f"Telling node to shutdown in {secs} seconds")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def getMetadata(self):
        """Get the node's metadata."""
        p = admin_pb2.AdminMessage()
        p.get_device_metadata_request = True
        logging.info(f"Requesting device metadata")

        self._sendAdmin(
            p, wantResponse=True, onResponse=self.onRequestGetMetadata
        )
        self.iface.waitForAckNak()

    def factoryReset(self, full: bool = False):
        """Tell the node to factory reset."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        if full:
            p.factory_reset_device = True
            logging.info(f"Telling node to factory reset (full device reset)")
        else:
            p.factory_reset_config = True
            logging.info(f"Telling node to factory reset (config reset)")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def removeNode(self, nodeId: Union[int, str]):
        """Tell the node to remove a specific node by ID"""
        self.ensureSessionKey()
        if isinstance(nodeId, str):
            if nodeId.startswith("!"):
                nodeId = int(nodeId[1:], 16)
            else:
                nodeId = int(nodeId)

        p = admin_pb2.AdminMessage()
        p.remove_by_nodenum = nodeId

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def setFavorite(self, nodeId: Union[int, str]):
        """Tell the node to set the specified node ID to be favorited on the NodeDB on the device"""
        self.ensureSessionKey()
        if isinstance(nodeId, str):
            if nodeId.startswith("!"):
                nodeId = int(nodeId[1:], 16)
            else:
                nodeId = int(nodeId)

        p = admin_pb2.AdminMessage()
        p.set_favorite_node = nodeId

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def removeFavorite(self, nodeId: Union[int, str]):
        """Tell the node to set the specified node ID to be un-favorited on the NodeDB on the device"""
        self.ensureSessionKey()
        if isinstance(nodeId, str):
            if nodeId.startswith("!"):
                nodeId = int(nodeId[1:], 16)
            else:
                nodeId = int(nodeId)

        p = admin_pb2.AdminMessage()
        p.remove_favorite_node = nodeId

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def setIgnored(self, nodeId: Union[int, str]):
        """Tell the node to set the specified node ID to be ignored on the NodeDB on the device"""
        self.ensureSessionKey()
        if isinstance(nodeId, str):
            if nodeId.startswith("!"):
                nodeId = int(nodeId[1:], 16)
            else:
                nodeId = int(nodeId)

        p = admin_pb2.AdminMessage()
        p.set_ignored_node = nodeId

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def removeIgnored(self, nodeId: Union[int, str]):
        """Tell the node to set the specified node ID to be un-ignored on the NodeDB on the device"""
        self.ensureSessionKey()
        if isinstance(nodeId, str):
            if nodeId.startswith("!"):
                nodeId = int(nodeId[1:], 16)
            else:
                nodeId = int(nodeId)

        p = admin_pb2.AdminMessage()
        p.remove_ignored_node = nodeId

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def resetNodeDb(self):
        """Tell the node to reset its list of nodes."""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.nodedb_reset = True
        logging.info(f"Telling node to reset the NodeDB")

        # If sending to a remote node, wait for ACK/NAK
        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def setFixedPosition(self, lat: Union[int, float], lon: Union[int, float], alt: int):
        """Tell the node to set fixed position to the provided value and enable the fixed position setting"""
        self.ensureSessionKey()

        p = mesh_pb2.Position()
        if isinstance(lat, float) and lat != 0.0:
            p.latitude_i = int(lat / 1e-7)
        elif isinstance(lat, int) and lat != 0:
            p.latitude_i = lat

        if isinstance(lon, float) and lon != 0.0:
            p.longitude_i = int(lon / 1e-7)
        elif isinstance(lon, int) and lon != 0:
            p.longitude_i = lon

        if alt != 0:
            p.altitude = alt

        a = admin_pb2.AdminMessage()
        a.set_fixed_position.CopyFrom(p)

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(a, onResponse=onResponse)

    def removeFixedPosition(self):
        """Tell the node to remove the fixed position and set the fixed position setting to false"""
        self.ensureSessionKey()
        p = admin_pb2.AdminMessage()
        p.remove_fixed_position = True
        logging.info(f"Telling node to remove fixed position")

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def setTime(self, timeSec: int = 0):
        """Tell the node to set its time to the provided timestamp, or the system's current time if not provided or 0."""
        self.ensureSessionKey()
        if timeSec == 0:
            timeSec = int(time.time())
        p = admin_pb2.AdminMessage()
        p.set_time_only = timeSec
        logging.info(f"Setting node time to {timeSec}")

        if self == self.iface.localNode:
            onResponse = None
        else:
            onResponse = self.onAckNak
        return self._sendAdmin(p, onResponse=onResponse)

    def _fixupChannels(self):
        """Fixup indexes and add disabled channels as needed"""

        # Add extra disabled channels as needed
        # This is needed because the protobufs will have index **missing** if the channel number is zero
        for index, ch in enumerate(self.channels):
            ch.index = index  # fixup indexes

        self._fillChannels()

    def _fillChannels(self):
        """Mark unused channels as disabled"""

        # Add extra disabled channels as needed
        index = len(self.channels)
        while index < 8:
            ch = channel_pb2.Channel()
            ch.role = channel_pb2.Channel.Role.DISABLED
            ch.index = index
            self.channels.append(ch)
            index += 1

    def onRequestGetMetadata(self, p):
        """Handle the response packet for requesting device metadata getMetadata()"""
        logging.debug(f"onRequestGetMetadata() p:{p}")

        if "routing" in p["decoded"]:
            if p["decoded"]["routing"]["errorReason"] != "NONE":
                print(f'Error on response: {p["decoded"]["routing"]["errorReason"]}')
                self.iface._acknowledgment.receivedNak = True
        else:
            self.iface._acknowledgment.receivedAck = True
            if p["decoded"]["portnum"] == portnums_pb2.PortNum.Name(
                portnums_pb2.PortNum.ROUTING_APP
            ):
                if p["decoded"]["routing"]["errorReason"] != "NONE":
                    logging.warning(
                        f'Metadata request failed, error reason: {p["decoded"]["routing"]["errorReason"]}'
                    )
                    self._timeout.expireTime = time.time()  # Do not wait any longer
                    return  # Don't try to parse this routing message
                logging.debug(f"Retrying metadata request.")
                self.getMetadata()
                return

            c = p["decoded"]["admin"]["raw"].get_device_metadata_response
            self._timeout.reset()  # We made forward progress
            logging.debug(f"Received metadata {stripnl(c)}")
            print(f"\nfirmware_version: {c.firmware_version}")
            print(f"device_state_version: {c.device_state_version}")

    def onResponseRequestChannel(self, p):
        """Handle the response packet for requesting a channel _requestChannel()"""
        logging.debug(f"onResponseRequestChannel() p:{p}")

        if p["decoded"]["portnum"] == portnums_pb2.PortNum.Name(
            portnums_pb2.PortNum.ROUTING_APP
        ):
            if p["decoded"]["routing"]["errorReason"] != "NONE":
                logging.warning(
                    f'Channel request failed, error reason: {p["decoded"]["routing"]["errorReason"]}'
                )
                self._timeout.expireTime = time.time()  # Do not wait any longer
                return  # Don't try to parse this routing message
            lastTried = 0
            if len(self.partialChannels) > 0:
                lastTried = self.partialChannels[-1].index
            logging.debug(f"Retrying previous channel request.")
            self._requestChannel(lastTried)
            return

        c = p["decoded"]["admin"]["raw"].get_channel_response
        self.partialChannels.append(c)
        self._timeout.reset()  # We made forward progress
        logging.debug(f"Received channel {stripnl(c)}")
        index = c.index

        if index >= 8 - 1:
            logging.debug("Finished downloading channels")

            self.channels = self.partialChannels
            self._fixupChannels()
        else:
            self._requestChannel(index + 1)

    def onAckNak(self, p):
        """Informative handler for ACK/NAK responses"""
        if p["decoded"]["routing"]["errorReason"] != "NONE":
            print(
                f'Received a NAK, error reason: {p["decoded"]["routing"]["errorReason"]}'
            )
            self.iface._acknowledgment.receivedNak = True
        else:
            if int(p["from"]) == self.iface.localNode.nodeNum:
                print(
                    f"Received an implicit ACK. Packet will likely arrive, but cannot be guaranteed."
                )
                self.iface._acknowledgment.receivedImplAck = True
            else:
                print(f"Received an ACK.")
                self.iface._acknowledgment.receivedAck = True

    def _requestChannel(self, channelNum: int):
        """Done with initial config messages, now send regular
        MeshPackets to ask for settings"""
        p = admin_pb2.AdminMessage()
        p.get_channel_request = channelNum + 1

        # Show progress message for super slow operations
        if self != self.iface.localNode:
            print(
                f"Requesting channel {channelNum} info from remote node (this could take a while)"
            )
            logging.debug(
                f"Requesting channel {channelNum} info from remote node (this could take a while)"
            )
        else:
            logging.debug(f"Requesting channel {channelNum}")

        return self._sendAdmin(
            p, wantResponse=True, onResponse=self.onResponseRequestChannel
        )

    # pylint: disable=R1710
    def _sendAdmin(
        self,
        p: admin_pb2.AdminMessage,
        wantResponse: bool=True,
        onResponse=None,
        adminIndex: int=0,
    ):
        """Send an admin message to the specified node (or the local node if destNodeNum is zero)"""

        if self.noProto:
            logging.warning(
                f"Not sending packet because protocol use is disabled by noProto"
            )
        else:
            if (
                adminIndex == 0
            ):  # unless a special channel index was used, we want to use the admin index
                adminIndex = self.iface.localNode._getAdminChannelIndex()
            logging.debug(f"adminIndex:{adminIndex}")
            if isinstance(self.nodeNum, int):
                nodeid = self.nodeNum
            else: # assume string starting with !
                nodeid = int(self.nodeNum[1:],16)
            if "adminSessionPassKey" in self.iface._getOrCreateByNum(nodeid):
                p.session_passkey = self.iface._getOrCreateByNum(nodeid).get("adminSessionPassKey")
            return self.iface.sendData(
                p,
                self.nodeNum,
                portNum=portnums_pb2.PortNum.ADMIN_APP,
                wantAck=False,
                wantResponse=wantResponse,
                onResponse=onResponse,
                channelIndex=adminIndex,
                pkiEncrypted=True,
            )

    def ensureSessionKey(self):
        """If our entry in iface.nodesByNum doesn't already have an adminSessionPassKey, make a request to get one"""
        if self.noProto:
            logging.warning(
                f"Not ensuring session key, because protocol use is disabled by noProto"
            )
        else:
            if isinstance(self.nodeNum, int):
                nodeid = self.nodeNum
            else: # assume string starting with !
                nodeid = int(self.nodeNum[1:],16)
            if self.iface._getOrCreateByNum(nodeid).get("adminSessionPassKey") is None:
                self.requestConfig(admin_pb2.AdminMessage.SESSIONKEY_CONFIG)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _RemoteHardwarePinType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _RemoteHardwarePinTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_RemoteHardwarePinType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _RemoteHardwarePinType.ValueType  # 0
    """
    Unset/unused
    """
    DIGITAL_READ: _RemoteHardwarePinType.ValueType  # 1
    """
    GPIO pin can be read (if it is high / low)
    """
    DIGITAL_WRITE: _RemoteHardwarePinType.ValueType  # 2
    """
    GPIO pin can be written to (high / low)
    """

class RemoteHardwarePinType(_RemoteHardwarePinType, metaclass=_RemoteHardwarePinTypeEnumTypeWrapper): ...

UNKNOWN: RemoteHardwarePinType.ValueType  # 0
"""
Unset/unused
"""
DIGITAL_READ: RemoteHardwarePinType.ValueType  # 1
"""
GPIO pin can be read (if it is high / low)
"""
DIGITAL_WRITE: RemoteHardwarePinType.ValueType  # 2
"""
GPIO pin can be written to (high / low)
"""
global___RemoteHardwarePinType = RemoteHardwarePinType

@typing.final
class ModuleConfig(google.protobuf.message.Message):
    """
    Module Config
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class MQTTConfig(google.protobuf.message.Message):
        """
        MQTT Client Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        ADDRESS_FIELD_NUMBER: builtins.int
        USERNAME_FIELD_NUMBER: builtins.int
        PASSWORD_FIELD_NUMBER: builtins.int
        ENCRYPTION_ENABLED_FIELD_NUMBER: builtins.int
        JSON_ENABLED_FIELD_NUMBER: builtins.int
        TLS_ENABLED_FIELD_NUMBER: builtins.int
        ROOT_FIELD_NUMBER: builtins.int
        PROXY_TO_CLIENT_ENABLED_FIELD_NUMBER: builtins.int
        MAP_REPORTING_ENABLED_FIELD_NUMBER: builtins.int
        MAP_REPORT_SETTINGS_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        If a meshtastic node is able to reach the internet it will normally attempt to gateway any channels that are marked as
        is_uplink_enabled or is_downlink_enabled.
        """
        address: builtins.str
        """
        The server to use for our MQTT global message gateway feature.
        If not set, the default server will be used
        """
        username: builtins.str
        """
        MQTT username to use (most useful for a custom MQTT server).
        If using a custom server, this will be honoured even if empty.
        If using the default server, this will only be honoured if set, otherwise the device will use the default username
        """
        password: builtins.str
        """
        MQTT password to use (most useful for a custom MQTT server).
        If using a custom server, this will be honoured even if empty.
        If using the default server, this will only be honoured if set, otherwise the device will use the default password
        """
        encryption_enabled: builtins.bool
        """
        Whether to send encrypted or decrypted packets to MQTT.
        This parameter is only honoured if you also set server
        (the default official mqtt.meshtastic.org server can handle encrypted packets)
        Decrypted packets may be useful for external systems that want to consume meshtastic packets
        """
        json_enabled: builtins.bool
        """
        Whether to send / consume json packets on MQTT
        """
        tls_enabled: builtins.bool
        """
        If true, we attempt to establish a secure connection using TLS
        """
        root: builtins.str
        """
        The root topic to use for MQTT messages. Default is "msh".
        This is useful if you want to use a single MQTT server for multiple meshtastic networks and separate them via ACLs
        """
        proxy_to_client_enabled: builtins.bool
        """
        If true, we can use the connected phone / client to proxy messages to MQTT instead of a direct connection
        """
        map_reporting_enabled: builtins.bool
        """
        If true, we will periodically report unencrypted information about our node to a map via MQTT
        """
        @property
        def map_report_settings(self) -> global___ModuleConfig.MapReportSettings:
            """
            Settings for reporting information about our node to a map via MQTT
            """

        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            address: builtins.str = ...,
            username: builtins.str = ...,
            password: builtins.str = ...,
            encryption_enabled: builtins.bool = ...,
            json_enabled: builtins.bool = ...,
            tls_enabled: builtins.bool = ...,
            root: builtins.str = ...,
            proxy_to_client_enabled: builtins.bool = ...,
            map_reporting_enabled: builtins.bool = ...,
            map_report_settings: global___ModuleConfig.MapReportSettings | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["map_report_settings", b"map_report_settings"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["address", b"address", "enabled", b"enabled", "encryption_enabled", b"encryption_enabled", "json_enabled", b"json_enabled", "map_report_settings", b"map_report_settings", "map_reporting_enabled", b"map_reporting_enabled", "password", b"password", "proxy_to_client_enabled", b"proxy_to_client_enabled", "root", b"root", "tls_enabled", b"tls_enabled", "username", b"username"]) -> None: ...

    @typing.final
    class MapReportSettings(google.protobuf.message.Message):
        """
        Settings for reporting unencrypted information about our node to a map via MQTT
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PUBLISH_INTERVAL_SECS_FIELD_NUMBER: builtins.int
        POSITION_PRECISION_FIELD_NUMBER: builtins.int
        publish_interval_secs: builtins.int
        """
        How often we should report our info to the map (in seconds)
        """
        position_precision: builtins.int
        """
        Bits of precision for the location sent (default of 32 is full precision).
        """
        def __init__(
            self,
            *,
            publish_interval_secs: builtins.int = ...,
            position_precision: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["position_precision", b"position_precision", "publish_interval_secs", b"publish_interval_secs"]) -> None: ...

    @typing.final
    class RemoteHardwareConfig(google.protobuf.message.Message):
        """
        RemoteHardwareModule Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        ALLOW_UNDEFINED_PIN_ACCESS_FIELD_NUMBER: builtins.int
        AVAILABLE_PINS_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Whether the Module is enabled
        """
        allow_undefined_pin_access: builtins.bool
        """
        Whether the Module allows consumers to read / write to pins not defined in available_pins
        """
        @property
        def available_pins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RemoteHardwarePin]:
            """
            Exposes the available pins to the mesh for reading and writing
            """

        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            allow_undefined_pin_access: builtins.bool = ...,
            available_pins: collections.abc.Iterable[global___RemoteHardwarePin] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["allow_undefined_pin_access", b"allow_undefined_pin_access", "available_pins", b"available_pins", "enabled", b"enabled"]) -> None: ...

    @typing.final
    class NeighborInfoConfig(google.protobuf.message.Message):
        """
        NeighborInfoModule Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        TRANSMIT_OVER_LORA_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Whether the Module is enabled
        """
        update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        Neighbor Info (minimum is 14400, i.e., 4 hours)
        """
        transmit_over_lora: builtins.bool
        """
        Whether in addition to sending it to MQTT and the PhoneAPI, our NeighborInfo should be transmitted over LoRa.
        Note that this is not available on a channel with default key and name.
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            update_interval: builtins.int = ...,
            transmit_over_lora: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "transmit_over_lora", b"transmit_over_lora", "update_interval", b"update_interval"]) -> None: ...

    @typing.final
    class DetectionSensorConfig(google.protobuf.message.Message):
        """
        Detection Sensor Module Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _TriggerType:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _TriggerTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ModuleConfig.DetectionSensorConfig._TriggerType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            LOGIC_LOW: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 0
            """Event is triggered if pin is low"""
            LOGIC_HIGH: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 1
            """Event is triggered if pin is high"""
            FALLING_EDGE: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 2
            """Event is triggered when pin goes high to low"""
            RISING_EDGE: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 3
            """Event is triggered when pin goes low to high"""
            EITHER_EDGE_ACTIVE_LOW: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 4
            """Event is triggered on every pin state change, low is considered to be
            "active"
            """
            EITHER_EDGE_ACTIVE_HIGH: ModuleConfig.DetectionSensorConfig._TriggerType.ValueType  # 5
            """Event is triggered on every pin state change, high is considered to be
            "active"
            """

        class TriggerType(_TriggerType, metaclass=_TriggerTypeEnumTypeWrapper): ...
        LOGIC_LOW: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 0
        """Event is triggered if pin is low"""
        LOGIC_HIGH: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 1
        """Event is triggered if pin is high"""
        FALLING_EDGE: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 2
        """Event is triggered when pin goes high to low"""
        RISING_EDGE: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 3
        """Event is triggered when pin goes low to high"""
        EITHER_EDGE_ACTIVE_LOW: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 4
        """Event is triggered on every pin state change, low is considered to be
        "active"
        """
        EITHER_EDGE_ACTIVE_HIGH: ModuleConfig.DetectionSensorConfig.TriggerType.ValueType  # 5
        """Event is triggered on every pin state change, high is considered to be
        "active"
        """

        ENABLED_FIELD_NUMBER: builtins.int
        MINIMUM_BROADCAST_SECS_FIELD_NUMBER: builtins.int
        STATE_BROADCAST_SECS_FIELD_NUMBER: builtins.int
        SEND_BELL_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        MONITOR_PIN_FIELD_NUMBER: builtins.int
        DETECTION_TRIGGER_TYPE_FIELD_NUMBER: builtins.int
        USE_PULLUP_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Whether the Module is enabled
        """
        minimum_broadcast_secs: builtins.int
        """
        Interval in seconds of how often we can send a message to the mesh when a
        trigger event is detected
        """
        state_broadcast_secs: builtins.int
        """
        Interval in seconds of how often we should send a message to the mesh
        with the current state regardless of trigger events When set to 0, only
        trigger events will be broadcasted Works as a sort of status heartbeat
        for peace of mind
        """
        send_bell: builtins.bool
        """
        Send ASCII bell with alert message
        Useful for triggering ext. notification on bell
        """
        name: builtins.str
        """
        Friendly name used to format message sent to mesh
        Example: A name "Motion" would result in a message "Motion detected"
        Maximum length of 20 characters
        """
        monitor_pin: builtins.int
        """
        GPIO pin to monitor for state changes
        """
        detection_trigger_type: global___ModuleConfig.DetectionSensorConfig.TriggerType.ValueType
        """
        The type of trigger event to be used
        """
        use_pullup: builtins.bool
        """
        Whether or not use INPUT_PULLUP mode for GPIO pin
        Only applicable if the board uses pull-up resistors on the pin
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            minimum_broadcast_secs: builtins.int = ...,
            state_broadcast_secs: builtins.int = ...,
            send_bell: builtins.bool = ...,
            name: builtins.str = ...,
            monitor_pin: builtins.int = ...,
            detection_trigger_type: global___ModuleConfig.DetectionSensorConfig.TriggerType.ValueType = ...,
            use_pullup: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["detection_trigger_type", b"detection_trigger_type", "enabled", b"enabled", "minimum_broadcast_secs", b"minimum_broadcast_secs", "monitor_pin", b"monitor_pin", "name", b"name", "send_bell", b"send_bell", "state_broadcast_secs", b"state_broadcast_secs", "use_pullup", b"use_pullup"]) -> None: ...

    @typing.final
    class AudioConfig(google.protobuf.message.Message):
        """
        Audio Config for codec2 voice
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Audio_Baud:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _Audio_BaudEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ModuleConfig.AudioConfig._Audio_Baud.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            CODEC2_DEFAULT: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 0
            CODEC2_3200: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 1
            CODEC2_2400: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 2
            CODEC2_1600: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 3
            CODEC2_1400: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 4
            CODEC2_1300: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 5
            CODEC2_1200: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 6
            CODEC2_700: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 7
            CODEC2_700B: ModuleConfig.AudioConfig._Audio_Baud.ValueType  # 8

        class Audio_Baud(_Audio_Baud, metaclass=_Audio_BaudEnumTypeWrapper):
            """
            Baudrate for codec2 voice
            """

        CODEC2_DEFAULT: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 0
        CODEC2_3200: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 1
        CODEC2_2400: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 2
        CODEC2_1600: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 3
        CODEC2_1400: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 4
        CODEC2_1300: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 5
        CODEC2_1200: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 6
        CODEC2_700: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 7
        CODEC2_700B: ModuleConfig.AudioConfig.Audio_Baud.ValueType  # 8

        CODEC2_ENABLED_FIELD_NUMBER: builtins.int
        PTT_PIN_FIELD_NUMBER: builtins.int
        BITRATE_FIELD_NUMBER: builtins.int
        I2S_WS_FIELD_NUMBER: builtins.int
        I2S_SD_FIELD_NUMBER: builtins.int
        I2S_DIN_FIELD_NUMBER: builtins.int
        I2S_SCK_FIELD_NUMBER: builtins.int
        codec2_enabled: builtins.bool
        """
        Whether Audio is enabled
        """
        ptt_pin: builtins.int
        """
        PTT Pin
        """
        bitrate: global___ModuleConfig.AudioConfig.Audio_Baud.ValueType
        """
        The audio sample rate to use for codec2
        """
        i2s_ws: builtins.int
        """
        I2S Word Select
        """
        i2s_sd: builtins.int
        """
        I2S Data IN
        """
        i2s_din: builtins.int
        """
        I2S Data OUT
        """
        i2s_sck: builtins.int
        """
        I2S Clock
        """
        def __init__(
            self,
            *,
            codec2_enabled: builtins.bool = ...,
            ptt_pin: builtins.int = ...,
            bitrate: global___ModuleConfig.AudioConfig.Audio_Baud.ValueType = ...,
            i2s_ws: builtins.int = ...,
            i2s_sd: builtins.int = ...,
            i2s_din: builtins.int = ...,
            i2s_sck: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["bitrate", b"bitrate", "codec2_enabled", b"codec2_enabled", "i2s_din", b"i2s_din", "i2s_sck", b"i2s_sck", "i2s_sd", b"i2s_sd", "i2s_ws", b"i2s_ws", "ptt_pin", b"ptt_pin"]) -> None: ...

    @typing.final
    class PaxcounterConfig(google.protobuf.message.Message):
        """
        Config for the Paxcounter Module
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        PAXCOUNTER_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        WIFI_THRESHOLD_FIELD_NUMBER: builtins.int
        BLE_THRESHOLD_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Enable the Paxcounter Module
        """
        paxcounter_update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        metrics to the mesh
        """
        wifi_threshold: builtins.int
        """
        WiFi RSSI threshold. Defaults to -80
        """
        ble_threshold: builtins.int
        """
        BLE RSSI threshold. Defaults to -80
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            paxcounter_update_interval: builtins.int = ...,
            wifi_threshold: builtins.int = ...,
            ble_threshold: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["ble_threshold", b"ble_threshold", "enabled", b"enabled", "paxcounter_update_interval", b"paxcounter_update_interval", "wifi_threshold", b"wifi_threshold"]) -> None: ...

    @typing.final
    class SerialConfig(google.protobuf.message.Message):
        """
        Serial Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Serial_Baud:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _Serial_BaudEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ModuleConfig.SerialConfig._Serial_Baud.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            BAUD_DEFAULT: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 0
            BAUD_110: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 1
            BAUD_300: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 2
            BAUD_600: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 3
            BAUD_1200: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 4
            BAUD_2400: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 5
            BAUD_4800: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 6
            BAUD_9600: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 7
            BAUD_19200: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 8
            BAUD_38400: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 9
            BAUD_57600: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 10
            BAUD_115200: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 11
            BAUD_230400: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 12
            BAUD_460800: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 13
            BAUD_576000: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 14
            BAUD_921600: ModuleConfig.SerialConfig._Serial_Baud.ValueType  # 15

        class Serial_Baud(_Serial_Baud, metaclass=_Serial_BaudEnumTypeWrapper):
            """
            TODO: REPLACE
            """

        BAUD_DEFAULT: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 0
        BAUD_110: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 1
        BAUD_300: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 2
        BAUD_600: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 3
        BAUD_1200: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 4
        BAUD_2400: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 5
        BAUD_4800: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 6
        BAUD_9600: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 7
        BAUD_19200: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 8
        BAUD_38400: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 9
        BAUD_57600: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 10
        BAUD_115200: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 11
        BAUD_230400: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 12
        BAUD_460800: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 13
        BAUD_576000: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 14
        BAUD_921600: ModuleConfig.SerialConfig.Serial_Baud.ValueType  # 15

        class _Serial_Mode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _Serial_ModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ModuleConfig.SerialConfig._Serial_Mode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DEFAULT: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 0
            SIMPLE: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 1
            PROTO: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 2
            TEXTMSG: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 3
            NMEA: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 4
            CALTOPO: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 5
            """NMEA messages specifically tailored for CalTopo"""
            WS85: ModuleConfig.SerialConfig._Serial_Mode.ValueType  # 6
            """Ecowitt WS85 weather station"""

        class Serial_Mode(_Serial_Mode, metaclass=_Serial_ModeEnumTypeWrapper):
            """
            TODO: REPLACE
            """

        DEFAULT: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 0
        SIMPLE: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 1
        PROTO: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 2
        TEXTMSG: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 3
        NMEA: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 4
        CALTOPO: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 5
        """NMEA messages specifically tailored for CalTopo"""
        WS85: ModuleConfig.SerialConfig.Serial_Mode.ValueType  # 6
        """Ecowitt WS85 weather station"""

        ENABLED_FIELD_NUMBER: builtins.int
        ECHO_FIELD_NUMBER: builtins.int
        RXD_FIELD_NUMBER: builtins.int
        TXD_FIELD_NUMBER: builtins.int
        BAUD_FIELD_NUMBER: builtins.int
        TIMEOUT_FIELD_NUMBER: builtins.int
        MODE_FIELD_NUMBER: builtins.int
        OVERRIDE_CONSOLE_SERIAL_PORT_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Preferences for the SerialModule
        """
        echo: builtins.bool
        """
        TODO: REPLACE
        """
        rxd: builtins.int
        """
        RX pin (should match Arduino gpio pin number)
        """
        txd: builtins.int
        """
        TX pin (should match Arduino gpio pin number)
        """
        baud: global___ModuleConfig.SerialConfig.Serial_Baud.ValueType
        """
        Serial baud rate
        """
        timeout: builtins.int
        """
        TODO: REPLACE
        """
        mode: global___ModuleConfig.SerialConfig.Serial_Mode.ValueType
        """
        Mode for serial module operation
        """
        override_console_serial_port: builtins.bool
        """
        Overrides the platform's defacto Serial port instance to use with Serial module config settings
        This is currently only usable in output modes like NMEA / CalTopo and may behave strangely or not work at all in other modes
        Existing logging over the Serial Console will still be present
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            echo: builtins.bool = ...,
            rxd: builtins.int = ...,
            txd: builtins.int = ...,
            baud: global___ModuleConfig.SerialConfig.Serial_Baud.ValueType = ...,
            timeout: builtins.int = ...,
            mode: global___ModuleConfig.SerialConfig.Serial_Mode.ValueType = ...,
            override_console_serial_port: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["baud", b"baud", "echo", b"echo", "enabled", b"enabled", "mode", b"mode", "override_console_serial_port", b"override_console_serial_port", "rxd", b"rxd", "timeout", b"timeout", "txd", b"txd"]) -> None: ...

    @typing.final
    class ExternalNotificationConfig(google.protobuf.message.Message):
        """
        External Notifications Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        OUTPUT_MS_FIELD_NUMBER: builtins.int
        OUTPUT_FIELD_NUMBER: builtins.int
        OUTPUT_VIBRA_FIELD_NUMBER: builtins.int
        OUTPUT_BUZZER_FIELD_NUMBER: builtins.int
        ACTIVE_FIELD_NUMBER: builtins.int
        ALERT_MESSAGE_FIELD_NUMBER: builtins.int
        ALERT_MESSAGE_VIBRA_FIELD_NUMBER: builtins.int
        ALERT_MESSAGE_BUZZER_FIELD_NUMBER: builtins.int
        ALERT_BELL_FIELD_NUMBER: builtins.int
        ALERT_BELL_VIBRA_FIELD_NUMBER: builtins.int
        ALERT_BELL_BUZZER_FIELD_NUMBER: builtins.int
        USE_PWM_FIELD_NUMBER: builtins.int
        NAG_TIMEOUT_FIELD_NUMBER: builtins.int
        USE_I2S_AS_BUZZER_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Enable the ExternalNotificationModule
        """
        output_ms: builtins.int
        """
        When using in On/Off mode, keep the output on for this many
        milliseconds. Default 1000ms (1 second).
        """
        output: builtins.int
        """
        Define the output pin GPIO setting Defaults to
        EXT_NOTIFY_OUT if set for the board.
        In standalone devices this pin should drive the LED to match the UI.
        """
        output_vibra: builtins.int
        """
        Optional: Define a secondary output pin for a vibra motor
        This is used in standalone devices to match the UI.
        """
        output_buzzer: builtins.int
        """
        Optional: Define a tertiary output pin for an active buzzer
        This is used in standalone devices to to match the UI.
        """
        active: builtins.bool
        """
        IF this is true, the 'output' Pin will be pulled active high, false
        means active low.
        """
        alert_message: builtins.bool
        """
        True: Alert when a text message arrives (output)
        """
        alert_message_vibra: builtins.bool
        """
        True: Alert when a text message arrives (output_vibra)
        """
        alert_message_buzzer: builtins.bool
        """
        True: Alert when a text message arrives (output_buzzer)
        """
        alert_bell: builtins.bool
        """
        True: Alert when the bell character is received (output)
        """
        alert_bell_vibra: builtins.bool
        """
        True: Alert when the bell character is received (output_vibra)
        """
        alert_bell_buzzer: builtins.bool
        """
        True: Alert when the bell character is received (output_buzzer)
        """
        use_pwm: builtins.bool
        """
        use a PWM output instead of a simple on/off output. This will ignore
        the 'output', 'output_ms' and 'active' settings and use the
        device.buzzer_gpio instead.
        """
        nag_timeout: builtins.int
        """
        The notification will toggle with 'output_ms' for this time of seconds.
        Default is 0 which means don't repeat at all. 60 would mean blink
        and/or beep for 60 seconds
        """
        use_i2s_as_buzzer: builtins.bool
        """
        When true, enables devices with native I2S audio output to use the RTTTL over speaker like a buzzer
        T-Watch S3 and T-Deck for example have this capability
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            output_ms: builtins.int = ...,
            output: builtins.int = ...,
            output_vibra: builtins.int = ...,
            output_buzzer: builtins.int = ...,
            active: builtins.bool = ...,
            alert_message: builtins.bool = ...,
            alert_message_vibra: builtins.bool = ...,
            alert_message_buzzer: builtins.bool = ...,
            alert_bell: builtins.bool = ...,
            alert_bell_vibra: builtins.bool = ...,
            alert_bell_buzzer: builtins.bool = ...,
            use_pwm: builtins.bool = ...,
            nag_timeout: builtins.int = ...,
            use_i2s_as_buzzer: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["active", b"active", "alert_bell", b"alert_bell", "alert_bell_buzzer", b"alert_bell_buzzer", "alert_bell_vibra", b"alert_bell_vibra", "alert_message", b"alert_message", "alert_message_buzzer", b"alert_message_buzzer", "alert_message_vibra", b"alert_message_vibra", "enabled", b"enabled", "nag_timeout", b"nag_timeout", "output", b"output", "output_buzzer", b"output_buzzer", "output_ms", b"output_ms", "output_vibra", b"output_vibra", "use_i2s_as_buzzer", b"use_i2s_as_buzzer", "use_pwm", b"use_pwm"]) -> None: ...

    @typing.final
    class StoreForwardConfig(google.protobuf.message.Message):
        """
        Store and Forward Module Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        HEARTBEAT_FIELD_NUMBER: builtins.int
        RECORDS_FIELD_NUMBER: builtins.int
        HISTORY_RETURN_MAX_FIELD_NUMBER: builtins.int
        HISTORY_RETURN_WINDOW_FIELD_NUMBER: builtins.int
        IS_SERVER_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Enable the Store and Forward Module
        """
        heartbeat: builtins.bool
        """
        TODO: REPLACE
        """
        records: builtins.int
        """
        TODO: REPLACE
        """
        history_return_max: builtins.int
        """
        TODO: REPLACE
        """
        history_return_window: builtins.int
        """
        TODO: REPLACE
        """
        is_server: builtins.bool
        """
        Set to true to let this node act as a server that stores received messages and resends them upon request.
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            heartbeat: builtins.bool = ...,
            records: builtins.int = ...,
            history_return_max: builtins.int = ...,
            history_return_window: builtins.int = ...,
            is_server: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "heartbeat", b"heartbeat", "history_return_max", b"history_return_max", "history_return_window", b"history_return_window", "is_server", b"is_server", "records", b"records"]) -> None: ...

    @typing.final
    class RangeTestConfig(google.protobuf.message.Message):
        """
        Preferences for the RangeTestModule
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ENABLED_FIELD_NUMBER: builtins.int
        SENDER_FIELD_NUMBER: builtins.int
        SAVE_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Enable the Range Test Module
        """
        sender: builtins.int
        """
        Send out range test messages from this node
        """
        save: builtins.bool
        """
        Bool value indicating that this node should save a RangeTest.csv file.
        ESP32 Only
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            sender: builtins.int = ...,
            save: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "save", b"save", "sender", b"sender"]) -> None: ...

    @typing.final
    class TelemetryConfig(google.protobuf.message.Message):
        """
        Configuration for both device and environment metrics
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        DEVICE_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        ENVIRONMENT_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        ENVIRONMENT_MEASUREMENT_ENABLED_FIELD_NUMBER: builtins.int
        ENVIRONMENT_SCREEN_ENABLED_FIELD_NUMBER: builtins.int
        ENVIRONMENT_DISPLAY_FAHRENHEIT_FIELD_NUMBER: builtins.int
        AIR_QUALITY_ENABLED_FIELD_NUMBER: builtins.int
        AIR_QUALITY_INTERVAL_FIELD_NUMBER: builtins.int
        POWER_MEASUREMENT_ENABLED_FIELD_NUMBER: builtins.int
        POWER_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        POWER_SCREEN_ENABLED_FIELD_NUMBER: builtins.int
        HEALTH_MEASUREMENT_ENABLED_FIELD_NUMBER: builtins.int
        HEALTH_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        HEALTH_SCREEN_ENABLED_FIELD_NUMBER: builtins.int
        device_update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        device metrics to the mesh
        """
        environment_update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        environment measurements to the mesh
        """
        environment_measurement_enabled: builtins.bool
        """
        Preferences for the Telemetry Module (Environment)
        Enable/Disable the telemetry measurement module measurement collection
        """
        environment_screen_enabled: builtins.bool
        """
        Enable/Disable the telemetry measurement module on-device display
        """
        environment_display_fahrenheit: builtins.bool
        """
        We'll always read the sensor in Celsius, but sometimes we might want to
        display the results in Fahrenheit as a "user preference".
        """
        air_quality_enabled: builtins.bool
        """
        Enable/Disable the air quality metrics
        """
        air_quality_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        air quality metrics to the mesh
        """
        power_measurement_enabled: builtins.bool
        """
        Enable/disable Power metrics
        """
        power_update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        power metrics to the mesh
        """
        power_screen_enabled: builtins.bool
        """
        Enable/Disable the power measurement module on-device display
        """
        health_measurement_enabled: builtins.bool
        """
        Preferences for the (Health) Telemetry Module
        Enable/Disable the telemetry measurement module measurement collection
        """
        health_update_interval: builtins.int
        """
        Interval in seconds of how often we should try to send our
        health metrics to the mesh
        """
        health_screen_enabled: builtins.bool
        """
        Enable/Disable the health telemetry module on-device display
        """
        def __init__(
            self,
            *,
            device_update_interval: builtins.int = ...,
            environment_update_interval: builtins.int = ...,
            environment_measurement_enabled: builtins.bool = ...,
            environment_screen_enabled: builtins.bool = ...,
            environment_display_fahrenheit: builtins.bool = ...,
            air_quality_enabled: builtins.bool = ...,
            air_quality_interval: builtins.int = ...,
            power_measurement_enabled: builtins.bool = ...,
            power_update_interval: builtins.int = ...,
            power_screen_enabled: builtins.bool = ...,
            health_measurement_enabled: builtins.bool = ...,
            health_update_interval: builtins.int = ...,
            health_screen_enabled: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["air_quality_enabled", b"air_quality_enabled", "air_quality_interval", b"air_quality_interval", "device_update_interval", b"device_update_interval", "environment_display_fahrenheit", b"environment_display_fahrenheit", "environment_measurement_enabled", b"environment_measurement_enabled", "environment_screen_enabled", b"environment_screen_enabled", "environment_update_interval", b"environment_update_interval", "health_measurement_enabled", b"health_measurement_enabled", "health_screen_enabled", b"health_screen_enabled", "health_update_interval", b"health_update_interval", "power_measurement_enabled", b"power_measurement_enabled", "power_screen_enabled", b"power_screen_enabled", "power_update_interval", b"power_update_interval"]) -> None: ...

    @typing.final
    class CannedMessageConfig(google.protobuf.message.Message):
        """
        Canned Messages Module Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _InputEventChar:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _InputEventCharEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ModuleConfig.CannedMessageConfig._InputEventChar.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            NONE: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 0
            """
            TODO: REPLACE
            """
            UP: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 17
            """
            TODO: REPLACE
            """
            DOWN: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 18
            """
            TODO: REPLACE
            """
            LEFT: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 19
            """
            TODO: REPLACE
            """
            RIGHT: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 20
            """
            TODO: REPLACE
            """
            SELECT: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 10
            """
            '\\n'
            """
            BACK: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 27
            """
            TODO: REPLACE
            """
            CANCEL: ModuleConfig.CannedMessageConfig._InputEventChar.ValueType  # 24
            """
            TODO: REPLACE
            """

        class InputEventChar(_InputEventChar, metaclass=_InputEventCharEnumTypeWrapper):
            """
            TODO: REPLACE
            """

        NONE: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 0
        """
        TODO: REPLACE
        """
        UP: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 17
        """
        TODO: REPLACE
        """
        DOWN: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 18
        """
        TODO: REPLACE
        """
        LEFT: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 19
        """
        TODO: REPLACE
        """
        RIGHT: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 20
        """
        TODO: REPLACE
        """
        SELECT: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 10
        """
        '\\n'
        """
        BACK: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 27
        """
        TODO: REPLACE
        """
        CANCEL: ModuleConfig.CannedMessageConfig.InputEventChar.ValueType  # 24
        """
        TODO: REPLACE
        """

        ROTARY1_ENABLED_FIELD_NUMBER: builtins.int
        INPUTBROKER_PIN_A_FIELD_NUMBER: builtins.int
        INPUTBROKER_PIN_B_FIELD_NUMBER: builtins.int
        INPUTBROKER_PIN_PRESS_FIELD_NUMBER: builtins.int
        INPUTBROKER_EVENT_CW_FIELD_NUMBER: builtins.int
        INPUTBROKER_EVENT_CCW_FIELD_NUMBER: builtins.int
        INPUTBROKER_EVENT_PRESS_FIELD_NUMBER: builtins.int
        UPDOWN1_ENABLED_FIELD_NUMBER: builtins.int
        ENABLED_FIELD_NUMBER: builtins.int
        ALLOW_INPUT_SOURCE_FIELD_NUMBER: builtins.int
        SEND_BELL_FIELD_NUMBER: builtins.int
        rotary1_enabled: builtins.bool
        """
        Enable the rotary encoder #1. This is a 'dumb' encoder sending pulses on both A and B pins while rotating.
        """
        inputbroker_pin_a: builtins.int
        """
        GPIO pin for rotary encoder A port.
        """
        inputbroker_pin_b: builtins.int
        """
        GPIO pin for rotary encoder B port.
        """
        inputbroker_pin_press: builtins.int
        """
        GPIO pin for rotary encoder Press port.
        """
        inputbroker_event_cw: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType
        """
        Generate input event on CW of this kind.
        """
        inputbroker_event_ccw: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType
        """
        Generate input event on CCW of this kind.
        """
        inputbroker_event_press: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType
        """
        Generate input event on Press of this kind.
        """
        updown1_enabled: builtins.bool
        """
        Enable the Up/Down/Select input device. Can be RAK rotary encoder or 3 buttons. Uses the a/b/press definitions from inputbroker.
        """
        enabled: builtins.bool
        """
        Enable/disable CannedMessageModule.
        """
        allow_input_source: builtins.str
        """
        Input event origin accepted by the canned message module.
        Can be e.g. "rotEnc1", "upDownEnc1", "scanAndSelect", "cardkb", "serialkb", or keyword "_any"
        """
        send_bell: builtins.bool
        """
        CannedMessageModule also sends a bell character with the messages.
        ExternalNotificationModule can benefit from this feature.
        """
        def __init__(
            self,
            *,
            rotary1_enabled: builtins.bool = ...,
            inputbroker_pin_a: builtins.int = ...,
            inputbroker_pin_b: builtins.int = ...,
            inputbroker_pin_press: builtins.int = ...,
            inputbroker_event_cw: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType = ...,
            inputbroker_event_ccw: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType = ...,
            inputbroker_event_press: global___ModuleConfig.CannedMessageConfig.InputEventChar.ValueType = ...,
            updown1_enabled: builtins.bool = ...,
            enabled: builtins.bool = ...,
            allow_input_source: builtins.str = ...,
            send_bell: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["allow_input_source", b"allow_input_source", "enabled", b"enabled", "inputbroker_event_ccw", b"inputbroker_event_ccw", "inputbroker_event_cw", b"inputbroker_event_cw", "inputbroker_event_press", b"inputbroker_event_press", "inputbroker_pin_a", b"inputbroker_pin_a", "inputbroker_pin_b", b"inputbroker_pin_b", "inputbroker_pin_press", b"inputbroker_pin_press", "rotary1_enabled", b"rotary1_enabled", "send_bell", b"send_bell", "updown1_enabled", b"updown1_enabled"]) -> None: ...

    @typing.final
    class AmbientLightingConfig(google.protobuf.message.Message):
        """
        Ambient Lighting Module - Settings for control of onboard LEDs to allow users to adjust the brightness levels and respective color levels.
        Initially created for the RAK14001 RGB LED module.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LED_STATE_FIELD_NUMBER: builtins.int
        CURRENT_FIELD_NUMBER: builtins.int
        RED_FIELD_NUMBER: builtins.int
        GREEN_FIELD_NUMBER: builtins.int
        BLUE_FIELD_NUMBER: builtins.int
        led_state: builtins.bool
        """
        Sets LED to on or off.
        """
        current: builtins.int
        """
        Sets the current for the LED output. Default is 10.
        """
        red: builtins.int
        """
        Sets the red LED level. Values are 0-255.
        """
        green: builtins.int
        """
        Sets the green LED level. Values are 0-255.
        """
        blue: builtins.int
        """
        Sets the blue LED level. Values are 0-255.
        """
        def __init__(
            self,
            *,
            led_state: builtins.bool = ...,
            current: builtins.int = ...,
            red: builtins.int = ...,
            green: builtins.int = ...,
            blue: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["blue", b"blue", "current", b"current", "green", b"green", "led_state", b"led_state", "red", b"red"]) -> None: ...

    MQTT_FIELD_NUMBER: builtins.int
    SERIAL_FIELD_NUMBER: builtins.int
    EXTERNAL_NOTIFICATION_FIELD_NUMBER: builtins.int
    STORE_FORWARD_FIELD_NUMBER: builtins.int
    RANGE_TEST_FIELD_NUMBER: builtins.int
    TELEMETRY_FIELD_NUMBER: builtins.int
    CANNED_MESSAGE_FIELD_NUMBER: builtins.int
    AUDIO_FIELD_NUMBER: builtins.int
    REMOTE_HARDWARE_FIELD_NUMBER: builtins.int
    NEIGHBOR_INFO_FIELD_NUMBER: builtins.int
    AMBIENT_LIGHTING_FIELD_NUMBER: builtins.int
    DETECTION_SENSOR_FIELD_NUMBER: builtins.int
    PAXCOUNTER_FIELD_NUMBER: builtins.int
    @property
    def mqtt(self) -> global___ModuleConfig.MQTTConfig:
        """
        TODO: REPLACE
        """

    @property
    def serial(self) -> global___ModuleConfig.SerialConfig:
        """
        TODO: REPLACE
        """

    @property
    def external_notification(self) -> global___ModuleConfig.ExternalNotificationConfig:
        """
        TODO: REPLACE
        """

    @property
    def store_forward(self) -> global___ModuleConfig.StoreForwardConfig:
        """
        TODO: REPLACE
        """

    @property
    def range_test(self) -> global___ModuleConfig.RangeTestConfig:
        """
        TODO: REPLACE
        """

    @property
    def telemetry(self) -> global___ModuleConfig.TelemetryConfig:
        """
        TODO: REPLACE
        """

    @property
    def canned_message(self) -> global___ModuleConfig.CannedMessageConfig:
        """
        TODO: REPLACE
        """

    @property
    def audio(self) -> global___ModuleConfig.AudioConfig:
        """
        TODO: REPLACE
        """

    @property
    def remote_hardware(self) -> global___ModuleConfig.RemoteHardwareConfig:
        """
        TODO: REPLACE
        """

    @property
    def neighbor_info(self) -> global___ModuleConfig.NeighborInfoConfig:
        """
        TODO: REPLACE
        """

    @property
    def ambient_lighting(self) -> global___ModuleConfig.AmbientLightingConfig:
        """
        TODO: REPLACE
        """

    @property
    def detection_sensor(self) -> global___ModuleConfig.DetectionSensorConfig:
        """
        TODO: REPLACE
        """

    @property
    def paxcounter(self) -> global___ModuleConfig.PaxcounterConfig:
        """
        TODO: REPLACE
        """

    def __init__(
        self,
        *,
        mqtt: global___ModuleConfig.MQTTConfig | None = ...,
        serial: global___ModuleConfig.SerialConfig | None = ...,
        external_notification: global___ModuleConfig.ExternalNotificationConfig | None = ...,
        store_forward: global___ModuleConfig.StoreForwardConfig | None = ...,
        range_test: global___ModuleConfig.RangeTestConfig | None = ...,
        telemetry: global___ModuleConfig.TelemetryConfig | None = ...,
        canned_message: global___ModuleConfig.CannedMessageConfig | None = ...,
        audio: global___ModuleConfig.AudioConfig | None = ...,
        remote_hardware: global___ModuleConfig.RemoteHardwareConfig | None = ...,
        neighbor_info: global___ModuleConfig.NeighborInfoConfig | None = ...,
        ambient_lighting: global___ModuleConfig.AmbientLightingConfig | None = ...,
        detection_sensor: global___ModuleConfig.DetectionSensorConfig | None = ...,
        paxcounter: global___ModuleConfig.PaxcounterConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ambient_lighting", b"ambient_lighting", "audio", b"audio", "canned_message", b"canned_message", "detection_sensor", b"detection_sensor", "external_notification", b"external_notification", "mqtt", b"mqtt", "neighbor_info", b"neighbor_info", "paxcounter", b"paxcounter", "payload_variant", b"payload_variant", "range_test", b"range_test", "remote_hardware", b"remote_hardware", "serial", b"serial", "store_forward", b"store_forward", "telemetry", b"telemetry"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ambient_lighting", b"ambient_lighting", "audio", b"audio", "canned_message", b"canned_message", "detection_sensor", b"detection_sensor", "external_notification", b"external_notification", "mqtt", b"mqtt", "neighbor_info", b"neighbor_info", "paxcounter", b"paxcounter", "payload_variant", b"payload_variant", "range_test", b"range_test", "remote_hardware", b"remote_hardware", "serial", b"serial", "store_forward", b"store_forward", "telemetry", b"telemetry"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["payload_variant", b"payload_variant"]) -> typing.Literal["mqtt", "serial", "external_notification", "store_forward", "range_test", "telemetry", "canned_message", "audio", "remote_hardware", "neighbor_info", "ambient_lighting", "detection_sensor", "paxcounter"] | None: ...

global___ModuleConfig = ModuleConfig

@typing.final
class RemoteHardwarePin(google.protobuf.message.Message):
    """
    A GPIO pin definition for remote hardware module
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GPIO_PIN_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    gpio_pin: builtins.int
    """
    GPIO Pin number (must match Arduino)
    """
    name: builtins.str
    """
    Name for the GPIO pin (i.e. Front gate, mailbox, etc)
    """
    type: global___RemoteHardwarePinType.ValueType
    """
    Type of GPIO access available to consumers on the mesh
    """
    def __init__(
        self,
        *,
        gpio_pin: builtins.int = ...,
        name: builtins.str = ...,
        type: global___RemoteHardwarePinType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["gpio_pin", b"gpio_pin", "name", b"name", "type", b"type"]) -> None: ...

global___RemoteHardwarePin = RemoteHardwarePin



"""TCPInterface class for interfacing with http endpoint
"""
# pylint: disable=R0917
import logging
import socket
from typing import Optional, cast

from meshtastic.stream_interface import StreamInterface

DEFAULT_TCP_PORT = 4403

class TCPInterface(StreamInterface):
    """Interface class for meshtastic devices over a TCP link"""

    def __init__(
        self,
        hostname: str,
        debugOut=None,
        noProto: bool=False,
        connectNow: bool=True,
        portNumber: int=DEFAULT_TCP_PORT,
        noNodes:bool=False,
    ):
        """Constructor, opens a connection to a specified IP address/hostname

        Keyword Arguments:
            hostname {string} -- Hostname/IP address of the device to connect to
        """

        self.stream = None

        self.hostname: str = hostname
        self.portNumber: int = portNumber

        self.socket: Optional[socket.socket] = None

        if connectNow:
            logging.debug(f"Connecting to {hostname}") # type: ignore[str-bytes-safe]
            server_address: tuple[str, int] = (hostname, portNumber)
            sock: Optional[socket.socket] = socket.create_connection(server_address)
            self.socket = sock
        else:
            self.socket = None

        StreamInterface.__init__(
            self, debugOut=debugOut, noProto=noProto, connectNow=connectNow, noNodes=noNodes
        )

    def _socket_shutdown(self) -> None:
        """Shutdown the socket.
        Note: Broke out this line so the exception could be unit tested.
        """
        if self.socket:		#mian: please check that this should be "if self.socket:"
            cast(socket.socket, self.socket).shutdown(socket.SHUT_RDWR)

    def myConnect(self) -> None:
        """Connect to socket"""
        server_address: tuple[str, int] = (self.hostname, self.portNumber)
        sock: Optional[socket.socket] = socket.create_connection(server_address)
        self.socket = sock

    def close(self) -> None:
        """Close a connection to the device"""
        logging.debug("Closing TCP stream")
        StreamInterface.close(self)
        # Sometimes the socket read might be blocked in the reader thread.
        # Therefore we force the shutdown by closing the socket here
        self._wantExit: bool = True
        if not self.socket is None:
            try:
                self._socket_shutdown()
            except:
                pass  # Ignore errors in shutdown, because we might have a race with the server
            self.socket.close()

    def _writeBytes(self, b: bytes) -> None:
        """Write an array of bytes to our stream and flush"""
        if self.socket:
            self.socket.send(b)

    def _readBytes(self, length) -> Optional[bytes]:
        """Read an array of bytes from our stream"""
        if self.socket:
            return self.socket.recv(length)
        else:
            return None



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ChannelSettings(google.protobuf.message.Message):
    """
    This information can be encoded as a QRcode/url so that other users can configure
    their radio to join the same channel.
    A note about how channel names are shown to users: channelname-X
    poundsymbol is a prefix used to indicate this is a channel name (idea from @professr).
    Where X is a letter from A-Z (base 26) representing a hash of the PSK for this
    channel - so that if the user changes anything about the channel (which does
    force a new PSK) this letter will also change. Thus preventing user confusion if
    two friends try to type in a channel name of "BobsChan" and then can't talk
    because their PSKs will be different.
    The PSK is hashed into this letter by "0x41 + [xor all bytes of the psk ] modulo 26"
    This also allows the option of someday if people have the PSK off (zero), the
    users COULD type in a channel name and be able to talk.
    FIXME: Add description of multi-channel support and how primary vs secondary channels are used.
    FIXME: explain how apps use channels for security.
    explain how remote settings and remote gpio are managed as an example
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHANNEL_NUM_FIELD_NUMBER: builtins.int
    PSK_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    UPLINK_ENABLED_FIELD_NUMBER: builtins.int
    DOWNLINK_ENABLED_FIELD_NUMBER: builtins.int
    MODULE_SETTINGS_FIELD_NUMBER: builtins.int
    channel_num: builtins.int
    """
    Deprecated in favor of LoraConfig.channel_num
    """
    psk: builtins.bytes
    """
    A simple pre-shared key for now for crypto.
    Must be either 0 bytes (no crypto), 16 bytes (AES128), or 32 bytes (AES256).
    A special shorthand is used for 1 byte long psks.
    These psks should be treated as only minimally secure,
    because they are listed in this source code.
    Those bytes are mapped using the following scheme:
    `0` = No crypto
    `1` = The special "default" channel key: {0xd4, 0xf1, 0xbb, 0x3a, 0x20, 0x29, 0x07, 0x59, 0xf0, 0xbc, 0xff, 0xab, 0xcf, 0x4e, 0x69, 0x01}
    `2` through 10 = The default channel key, except with 1 through 9 added to the last byte.
    Shown to user as simple1 through 10
    """
    name: builtins.str
    """
    A SHORT name that will be packed into the URL.
    Less than 12 bytes.
    Something for end users to call the channel
    If this is the empty string it is assumed that this channel
    is the special (minimally secure) "Default"channel.
    In user interfaces it should be rendered as a local language translation of "X".
    For channel_num hashing empty string will be treated as "X".
    Where "X" is selected based on the English words listed above for ModemPreset
    """
    id: builtins.int
    """
    Used to construct a globally unique channel ID.
    The full globally unique ID will be: "name.id" where ID is shown as base36.
    Assuming that the number of meshtastic users is below 20K (true for a long time)
    the chance of this 64 bit random number colliding with anyone else is super low.
    And the penalty for collision is low as well, it just means that anyone trying to decrypt channel messages might need to
    try multiple candidate channels.
    Any time a non wire compatible change is made to a channel, this field should be regenerated.
    There are a small number of 'special' globally known (and fairly) insecure standard channels.
    Those channels do not have a numeric id included in the settings, but instead it is pulled from
    a table of well known IDs.
    (see Well Known Channels FIXME)
    """
    uplink_enabled: builtins.bool
    """
    If true, messages on the mesh will be sent to the *public* internet by any gateway ndoe
    """
    downlink_enabled: builtins.bool
    """
    If true, messages seen on the internet will be forwarded to the local mesh.
    """
    @property
    def module_settings(self) -> global___ModuleSettings:
        """
        Per-channel module settings.
        """

    def __init__(
        self,
        *,
        channel_num: builtins.int = ...,
        psk: builtins.bytes = ...,
        name: builtins.str = ...,
        id: builtins.int = ...,
        uplink_enabled: builtins.bool = ...,
        downlink_enabled: builtins.bool = ...,
        module_settings: global___ModuleSettings | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["module_settings", b"module_settings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel_num", b"channel_num", "downlink_enabled", b"downlink_enabled", "id", b"id", "module_settings", b"module_settings", "name", b"name", "psk", b"psk", "uplink_enabled", b"uplink_enabled"]) -> None: ...

global___ChannelSettings = ChannelSettings

@typing.final
class ModuleSettings(google.protobuf.message.Message):
    """
    This message is specifically for modules to store per-channel configuration data.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POSITION_PRECISION_FIELD_NUMBER: builtins.int
    IS_CLIENT_MUTED_FIELD_NUMBER: builtins.int
    position_precision: builtins.int
    """
    Bits of precision for the location sent in position packets.
    """
    is_client_muted: builtins.bool
    """
    Controls whether or not the phone / clients should mute the current channel
    Useful for noisy public channels you don't necessarily want to disable
    """
    def __init__(
        self,
        *,
        position_precision: builtins.int = ...,
        is_client_muted: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["is_client_muted", b"is_client_muted", "position_precision", b"position_precision"]) -> None: ...

global___ModuleSettings = ModuleSettings

@typing.final
class Channel(google.protobuf.message.Message):
    """
    A pair of a channel number, mode and the (sharable) settings for that channel
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Role:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _RoleEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Channel._Role.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DISABLED: Channel._Role.ValueType  # 0
        """
        This channel is not in use right now
        """
        PRIMARY: Channel._Role.ValueType  # 1
        """
        This channel is used to set the frequency for the radio - all other enabled channels must be SECONDARY
        """
        SECONDARY: Channel._Role.ValueType  # 2
        """
        Secondary channels are only used for encryption/decryption/authentication purposes.
        Their radio settings (freq etc) are ignored, only psk is used.
        """

    class Role(_Role, metaclass=_RoleEnumTypeWrapper):
        """
        How this channel is being used (or not).
        Note: this field is an enum to give us options for the future.
        In particular, someday we might make a 'SCANNING' option.
        SCANNING channels could have different frequencies and the radio would
        occasionally check that freq to see if anything is being transmitted.
        For devices that have multiple physical radios attached, we could keep multiple PRIMARY/SCANNING channels active at once to allow
        cross band routing as needed.
        If a device has only a single radio (the common case) only one channel can be PRIMARY at a time
        (but any number of SECONDARY channels can't be sent received on that common frequency)
        """

    DISABLED: Channel.Role.ValueType  # 0
    """
    This channel is not in use right now
    """
    PRIMARY: Channel.Role.ValueType  # 1
    """
    This channel is used to set the frequency for the radio - all other enabled channels must be SECONDARY
    """
    SECONDARY: Channel.Role.ValueType  # 2
    """
    Secondary channels are only used for encryption/decryption/authentication purposes.
    Their radio settings (freq etc) are ignored, only psk is used.
    """

    INDEX_FIELD_NUMBER: builtins.int
    SETTINGS_FIELD_NUMBER: builtins.int
    ROLE_FIELD_NUMBER: builtins.int
    index: builtins.int
    """
    The index of this channel in the channel table (from 0 to MAX_NUM_CHANNELS-1)
    (Someday - not currently implemented) An index of -1 could be used to mean "set by name",
    in which case the target node will find and set the channel by settings.name.
    """
    role: global___Channel.Role.ValueType
    """
    TODO: REPLACE
    """
    @property
    def settings(self) -> global___ChannelSettings:
        """
        The new settings, or NULL to disable that channel
        """

    def __init__(
        self,
        *,
        index: builtins.int = ...,
        settings: global___ChannelSettings | None = ...,
        role: global___Channel.Role.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["settings", b"settings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["index", b"index", "role", b"role", "settings", b"settings"]) -> None: ...

global___Channel = Channel



"""With two radios connected serially, send and receive test
   messages and report back if successful.
"""
import logging
import sys
import time
import traceback
import io

from typing import List, Optional

from dotmap import DotMap # type: ignore[import-untyped]
from pubsub import pub # type: ignore[import-untyped]

import meshtastic.util
from meshtastic import BROADCAST_NUM
from meshtastic.serial_interface import SerialInterface
from meshtastic.tcp_interface import TCPInterface

"""The interfaces we are using for our tests"""
interfaces: List = []

"""A list of all packets we received while the current test was running"""
receivedPackets: Optional[List] = None

testsRunning: bool = False

testNumber: int = 0

sendingInterface = None


def onReceive(packet, interface) -> None:
    """Callback invoked when a packet arrives"""
    if sendingInterface == interface:
        pass
        # print("Ignoring sending interface")
    else:
        # print(f"From {interface.stream.port}: {packet}")
        p = DotMap(packet)

        if p.decoded.portnum == "TEXT_MESSAGE_APP":
            # We only care a about clear text packets
            if receivedPackets is not None:
                receivedPackets.append(p)


def onNode(node) -> None:
    """Callback invoked when the node DB changes"""
    print(f"Node changed: {node}")


def subscribe() -> None:
    """Subscribe to the topics the user probably wants to see, prints output to stdout"""

    pub.subscribe(onNode, "meshtastic.node")


def testSend(
    fromInterface, toInterface, isBroadcast: bool=False, asBinary: bool=False, wantAck: bool=False
) -> bool:
    """
    Sends one test packet between two nodes and then returns success or failure

    Arguments:
        fromInterface {[type]} -- [description]
        toInterface {[type]} -- [description]

    Returns:
        boolean -- True for success
    """
    # pylint: disable=W0603
    global receivedPackets
    receivedPackets = []
    fromNode = fromInterface.myInfo.my_node_num

    if isBroadcast:
        toNode = BROADCAST_NUM
    else:
        toNode = toInterface.myInfo.my_node_num

    logging.debug(f"Sending test wantAck={wantAck} packet from {fromNode} to {toNode}")
    # pylint: disable=W0603
    global sendingInterface
    sendingInterface = fromInterface
    if not asBinary:
        fromInterface.sendText(f"Test {testNumber}", toNode, wantAck=wantAck)
    else:
        fromInterface.sendData(
            (f"Binary {testNumber}").encode("utf-8"), toNode, wantAck=wantAck
        )
    for _ in range(60):  # max of 60 secs before we timeout
        time.sleep(1)
        if len(receivedPackets) >= 1:
            return True
    return False  # Failed to send


def runTests(numTests: int=50, wantAck: bool=False, maxFailures: int=0) -> bool:
    """Run the tests."""
    logging.info(f"Running {numTests} tests with wantAck={wantAck}")
    numFail: int = 0
    numSuccess: int = 0
    for _ in range(numTests):
        # pylint: disable=W0603
        global testNumber
        testNumber = testNumber + 1
        isBroadcast:bool = True
        # asBinary=(i % 2 == 0)
        success = testSend(
            interfaces[0], interfaces[1], isBroadcast, asBinary=False, wantAck=wantAck
        )
        if not success:
            numFail = numFail + 1
            logging.error(
                f"Test {testNumber} failed, expected packet not received ({numFail} failures so far)"
            )
        else:
            numSuccess = numSuccess + 1
            logging.info(
                f"Test {testNumber} succeeded {numSuccess} successes {numFail} failures so far"
            )

        time.sleep(1)

    if numFail > maxFailures:
        logging.error("Too many failures! Test failed!")
        return False
    return True


def testThread(numTests=50) -> bool:
    """Test thread"""
    logging.info("Found devices, starting tests...")
    result: bool = runTests(numTests, wantAck=True)
    if result:
        # Run another test
        # Allow a few dropped packets
        result = runTests(numTests, wantAck=False, maxFailures=1)
    return result


def onConnection(topic=pub.AUTO_TOPIC) -> None:
    """Callback invoked when we connect/disconnect from a radio"""
    print(f"Connection changed: {topic.getName()}")


def openDebugLog(portName) -> io.TextIOWrapper:
    """Open the debug log file"""
    debugname = "log" + portName.replace("/", "_")
    logging.info(f"Writing serial debugging to {debugname}")
    return open(debugname, "w+", buffering=1, encoding="utf8")


def testAll(numTests: int=5) -> bool:
    """
    Run a series of tests using devices we can find.
    This is called from the cli with the "--test" option.

    """
    ports: List[str] = meshtastic.util.findPorts(True)
    if len(ports) < 2:
        meshtastic.util.our_exit(
            "Warning: Must have at least two devices connected to USB."
        )

    pub.subscribe(onConnection, "meshtastic.connection")
    pub.subscribe(onReceive, "meshtastic.receive")
    # pylint: disable=W0603
    global interfaces
    interfaces = list(
        map(
            lambda port: SerialInterface(
                port, debugOut=openDebugLog(port), connectNow=True
            ),
            ports,
        )
    )

    logging.info("Ports opened, starting test")
    result: bool = testThread(numTests)

    for i in interfaces:
        i.close()

    return result


def testSimulator() -> None:
    """
    Assume that someone has launched meshtastic-native as a simulated node.
    Talk to that node over TCP, do some operations and if they are successful
    exit the process with a success code, else exit with a non zero exit code.

    Run with
    python3 -c 'from meshtastic.test import testSimulator; testSimulator()'
    """
    logging.basicConfig(level=logging.DEBUG)
    logging.info("Connecting to simulator on localhost!")
    try:
        iface: meshtastic.tcp_interface.TCPInterface = TCPInterface("localhost")
        iface.showInfo()
        iface.localNode.showInfo()
        iface.localNode.exitSimulator()
        iface.close()
        logging.info("Integration test successful!")
    except:
        print("Error while testing simulator:", sys.exc_info()[0])
        traceback.print_exc()
        sys.exit(1)
    sys.exit(0)
